#!/usr/bin/env python
# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import gc
import os
import gtk
import sys
import time
import thread
import gobject
import gettext
import weakref
import traceback

import Theme
import Avatar
import Config
import desktop
import Dialogs
import TrayIcon
import MainWindow
import StatusMenu
import SlashCommands
import PluginManager
import PreferenceWindow
import PluginManagerDialog
import ConversationManager
import ConversationLayoutManager
from Parser import UnifiedParser
from AvatarChooser import AvatarChooser
from CustomEmoticons import CustomEmoticons

import emesenelib.common
from emesenelib import core
from emesenelib import Socket
from emesenelib import Hotmail
from emesenecommon import *

gettext.install('emesene')
gettext.install('emesene', PATH.LANG_PATH, unicode=False)

SERVER_HOST = 'messenger.hotmail.com'
SERVER_IP = '65.54.239.80'

class Controller( gobject.GObject ):
    '''The Controller class concentrate all the logic of the program
    leaving the other classes to only implement the GUI.
    All the classes with GUI should receive a Controller instance to
    be able to do things.'''
    
    # suprise, most of these signals should be refactored
    __gsignals__ = {
        #'font-changed', font, bold, italic, pangoDesc.get_size() / pango.SCALE
        'font-changed' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, 
        (gobject.TYPE_STRING,gobject.TYPE_BOOLEAN,gobject.TYPE_BOOLEAN,gobject.TYPE_INT)),
        #'color-changed', '#%02X%02X%02X' % (red, green, blue)
        'color-changed' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, 
        (gobject.TYPE_STRING,)),
        'input-format-changed' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, 
        (gobject.TYPE_PYOBJECT,)),
        'avatar-changed' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, 
        ()),
        'preferences-changed' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, 
        ()),

    }

    def __init__( self, username, minimized, safe, leakdebug ):
        '''Constructor'''

        gobject.GObject.__init__(self)

        self.status_ordered = [
            [ 'NLN', 'AWY', 'BSY', 'BRB', 'PHN',
             'LUN', 'HDN', 'IDL', 'FLN' ],

            [ 'online', 'away', 'busy', 'brb', 'phone',
             'lunch', 'invisible', 'idle', 'offline' ],

            [ _( 'Online' ), _( 'Away' ), _( 'Busy' ), _( 'Be right back' ),
             _( 'On the phone' ), _( 'Gone to lunch' ), _( 'Invisible' ),
             _( 'Idle' ), _( 'Offline' ) ],
            
            [ _( '_Online' ), _( '_Away' ), _( '_Busy' ), _( 'Be _right back' ),
             _( 'On the _phone' ), _( 'Gone to _lunch' ), _( '_Invisible' ),
             _( 'I_dle' ), _( 'O_ffline' ) ]
        ]

        
        self.connected = False
        self.userEmail = None
        self.userStatus = None

        self.config = Config.Config()
        self.config.connect('change::debug', self.updateDebug)
        self.config.connect('change::binary', self.updateDebug)

        self.theme = Theme.Theme( self.config, 'default', 'default' )
        self.mainWindow = MainWindow.MainWindow( self )
        self.unifiedParser = UnifiedParser( self.theme )
        
        self.conversationLayoutManager = \
            ConversationLayoutManager.ConversationLayoutManager(self)
        self.conversationManager = \
            ConversationManager.ConversationManager(self)
                
        if not TrayIcon.disabled:
            self.trayIcon = TrayIcon.TrayIcon( self )

        if TrayIcon.disabled or not minimized:
            self.mainWindow.show()
        
        self.pluginManager = None
        
        gtk.gdk.set_program_class('emesene')
        gtk.window_set_default_icon_list( \
            self.theme.getImage( 'icon16' ), self.theme.getImage( 'icon32' ),
            self.theme.getImage( 'icon48' ), self.theme.getImage( 'icon96' ) )

        self.autoReplyMessage = ''
        self.msn = None
        self.lastILN = 0
        self.pendingAvatars = {}
        
        self.trayDisconnect = None
        self.traySeparator = None
        self.trayStatusMenu = None
        
        self.avatar = None
        self.cancel = False
        self.mediaEnabled = False
        self.prefsOpen = False

        self.safemode = safe
        self.leakdebug = leakdebug
        
        if username in self.mainWindow.login.users.getUsers():
            user = username
            self.mainWindow.login.getPreferences(user)
            pwd = self.mainWindow.login.getPass()
            status = self.mainWindow.login.getStatus()
        elif self.config.glob['autoLogin']:
            user = self.mainWindow.login.getUser()
            pwd = self.mainWindow.login.getPass()
            status = self.mainWindow.login.getStatus()
        else:
            user = ''
            pwd = ''
        
        if user != '' and pwd != '':
            self.login( user, pwd, status )

        gobject.timeout_add( 10000, self.collect )

    def collect( self, *args ):
        gc.collect()
        return True

    def getContactManager(self):
        return (None, self.msn.contactManager)[self.msn != None]

    def login( self, user, password, status ):
        '''do the login'''
        global loginStatus
        
        reconnecting = (self.mainWindow.currentInterface == 'reconnect')
        
        self.cancel = False
        self.userEmail = user
        self.userStatus = status
        self.mainWindow.buildInterface( 'loading' )
        self.config.setCurrentUser(user)

        if not self.config.glob['debug']:
            print "If you are reading this, you may want to enable debug"
            print "It's the first option in the connection tab in preferences"
        
        if self.config.glob['nodns']:
            server = SERVER_IP
        else:
            server = SERVER_HOST
        
        proxy = None
        
        if self.config.glob['httpMethod']:
            if self.config.glob['useProxy']:
                proxy = Socket.Proxy( self.config.glob['proxyHost'], \
                    self.config.glob['proxyPort'] )
            else:
                proxy = Socket.Proxy()
            

        # messenger.hotmail.com
        self.msn = core.Msnp( server, 1863, user, password,\
             self.config.getUserConfigPath(), proxy, self.safemode )
        self.msn.setDebug(self.config.glob['debug'], self.config.glob['binary'])
        
        if self.cancel == True:
            self.config.setCurrentUser('')
            del self.msn
            return
        
        thread.start_new_thread( login , ( self.msn , ) )

        j = 1
        while loginStatus == 0:
            time.sleep( 0.1 )
            if self.cancel == True:
                self.config.setCurrentUser('')
                return
            #the next few lines create the animation effect to the trayicon
            if not TrayIcon.disabled:
                if (j % 10) >= 5:
                    self.trayIcon.update( 'disconnected' )
                else:
                    self.trayIcon.update( 'login' )
                j += 1
            gtk.main_iteration( True )

        if loginStatus == -1:
            if not reconnecting:
                text = _( 'Error during login, please retry' )
                if loginError:
                    text = text + '\n(' + str(loginError) + ')'
                Dialogs.Error( self.mainWindow, text ).run()
                
                self.mainWindow.buildInterface( 'login' )
                loginStatus = 0
                self.config.setCurrentUser('')
                del self.msn
                return
            else:
                loginStatus = 0
                self.autoReconnect(user, password, status)
                return
        
        if self.cancel == True:
            self.config.setCurrentUser('')
            try:
                self.msn.logout()
            except Exception, e:
                print "Exception at logout", e
            self.msn = None
            return

        self.connected = True
        self.pendingAvatars = {}
        self.idleTimeout = 0

        self.themeChanged()
        self.msn.changeStatus( status )

        self.msn.connect( 'disconnected', self.disconnected )
        self.msn.connect( 'connection-closed', self.connectionClosed )
        self.msn.connect( 'error', self.onMsnpError )
        self.msn.connect( 'exception', self.onMsnpException )
        
        self.mainWindow.buildInterface( 'userlist' )
        self.mainWindow.userList.connect( 'item-selected', self.onItemSelected )
        self.changeAvatar( self.config.user['avatarPath'] )

        self.config.user['autoReply'] = False
        self.autoReplyMessage = self.config.user['autoReplyMessage']
        self.mediaEnabled = self.config.user['mediaEnabled']

        layout = self.config.user['conversationLayout']
        if not self.conversationLayoutManager.load(layout):
            self.conversationLayoutManager.setDefault()
        
        # menu items only available when connected
        if not self.trayStatusMenu and not TrayIcon.disabled:
            statusMenuItem = gtk.MenuItem( _( '_Status' ) )
            statusMenu = StatusMenu.StatusMenu( self )
            statusMenuItem.set_submenu( statusMenu )
            separator = gtk.SeparatorMenuItem()
            disconnect = gtk.ImageMenuItem( gtk.STOCK_DISCONNECT )
            disconnect.connect( 'activate', self.on_tray_disconnect )
            
            self.trayDisconnect = disconnect
            self.traySeparator = separator
            self.trayStatusMenu = statusMenuItem
        
        #Slash Command Handler
        self.Slash = SlashCommands.SlashCommands( self )
        
        # add here
        
        if not TrayIcon.disabled:
            # REMEMBER to remove them when disconnect    
            self.trayIcon.menu.prepend( self.trayDisconnect )
            self.trayIcon.menu.prepend( self.traySeparator )
            self.trayIcon.menu.prepend( self.trayStatusMenu )
            self.trayIcon.menu.show_all()
            self.trayIcon.update( self.msn.status )
        
        self.pluginManager = PluginManager.PluginManager( self )
        
        plugins = self.config.user['activePlugins'].split( ',' )
        
        self.startPlugins( plugins )
        
        self.msn.connect( 'initial-status-change', self.onInitialStatusChange )
        self.msn.connect( 'display-picture-changed', self.onDisplayPictureChanged )
        self.msn.connect( 'new-conversation', self.newConversation )
        self.msn.connect( 'user-attr-changed', self.onUserAttrChanged )
        self.msn.connect( 'group-attr-changed', self.onGroupAttrChanged )
        self.msn.connect( 'add-notification', self.addNotification )
        #self.msn.connect( 'remove-notification', self.removeNotification )
        self.msn.connect( 'user-list-change', self.refreshUserList )
        self.msn.connect( 'user-disconnected', self.userDisconnected )
        self.msn.connect( 'connection-problem', self.connectionProblem )
        
        self.msn.connect( 'offline-message-waiting', self.offlineMessageWaiting )
        self.msn.connect( 'offline-message-received', self.offlineMessageReceived )
        self.msn.connect( 'send-message-error', self.messageError )
        self.msn.connect( 'msnobj-changed', self.msnobjChanged )
        
        self.hotmail = Hotmail.Hotmail( self.msn, self.config )
        
        self.customEmoticons = CustomEmoticons( self.config, self )
        
        self.conversationManager.handleLogin( user )
        
        gobject.idle_add(self.checkPending)
        self.cancel = False

    def disconnected( self, msnp ):
        '''called when a error occurs and we cant keep connected.'''
        self.logout( False, True )
        
    def connectionClosed( self, msnp ):
        '''called when a connection error happens'''
        self.logout( False, True )

    def onMsnpError( self, msnp, error, description ):
        '''called when there is an error during some msnp operation'''
        Dialogs.Error( self.mainWindow , description ).run()

    def onMsnpException( self, msnp, exception ):
        '''called when there is an error in msn.process'''
        lines = traceback.format_exception(*exception)
        Dialogs.Exception( self.mainWindow, lines ).run()

    def onUserAttrChanged( self, userlist, contact ):
        if not self.msn.socket.onLogin:
            self.mainWindow.userList.updateContact( contact )

    def onDisplayPictureChanged( self, msnp, switchboard, msnobj, email ):
        if not self.msn.socket.onLogin:
            contact = self.msn.contactManager.getContact( email )
            self.mainWindow.userList.updateContact( contact )
        
    def onGroupAttrChanged( self, msnp, oldGroup, newGroup ):
        # TODO: make updateGroup work
        self.mainWindow.userList.updateGroup( oldGroup, newGroup )

    def onItemSelected( self, userlist, t ):
        
        if t != None:
            (objType, obj) = t
            
            if objType == 'user':
                self.newConversation( self.msn, obj.email, None, True )
    
    def autoReconnect( self, user, password, status ):
        self.mainWindow.buildInterface( 'reconnect' )
        self.mainWindow.login.setFieldValues(user, password, status)

        self.reconnectAfter = 30
        self.reconnectTimerId = gobject.timeout_add( 1000, \
            self.updateReconnectTimer )
        
        self.updateReconnectTimer()
    
    def updateReconnectTimer( self ):
        self.reconnectAfter -= 1
        self.mainWindow.login.setReconnectCounter( self.reconnectAfter )
        
        if self.reconnectAfter == 0:
            self.mainWindow.login.login()
            return False
        else:
            return True
    
    def cancelReconnect( self ):
        gobject.source_remove( self.reconnectTimerId )
        self.reconnectAfter = None
        self.mainWindow.buildInterface('login')
    
    def logout( self, closeConversations = True, autoReconnect = False ):
        '''do the logout'''
        global loginStatus
        
        if closeConversations:
            self.conversationManager.closeAll()
        else:
            self.conversationManager.disableAll()

        try:
            self.pluginManager.destroy()
        except:
            pass

        self.pluginManager = None
        
        # cleanup modules that require msn
        self.hotmail = None
        self.customEmoticons = None
        self.Slash.unregister_slash()
        self.Slash = None

        self.mainWindow.buildInterface( 'login' )
        
        if autoReconnect:
            status = emesenelib.common.reverse_status[self.msn.status]
            self.autoReconnect(self.msn.user, self.msn.password, status)
        
        self.connected = False
        
        if not TrayIcon.disabled:
            try:
                self.trayIcon.menu.remove( self.trayDisconnect )
                self.trayIcon.menu.remove( self.traySeparator )
                self.trayIcon.menu.remove( self.trayStatusMenu )
                self.trayIcon.update( 'disconnected' )
            except:
                pass
        
        self.msn.logout()

        weakmsn = weakref.ref(self.msn)
        self.msn = None

        if weakmsn() != None:
            print "warning: there are still some references to emesenelib"

        if self.leakdebug:
            print "references to msn:", sys.getrefcount(weakmsn()) -1
            print "cheat sheet: r(o) == gc.get_referrers(o)"
            print "  to create a msn reference: weakmsn()"
            print
            import code, readline, gc
            r = gc.get_referrers
            code.InteractiveConsole(locals()).interact( \
                "Entering controller.msn leak debug console")
            
        loginStatus = 0
        
    def cancelLogin(self):
        self.cancel = True
        self.goToLoginWindow()
        
    def goToLoginWindow( self ):
        global loginStatus
        self.mainWindow.buildInterface( 'login' )
        self.msn = None
        self.pluginManager = None
        loginStatus = 0
        
    def quit( self, status ):
        '''close the window, and do all the things needed...'''
        
        # makes it more responsive to the user by effectively hiding the
        # the window before disconnecting and quitting
        #self.mainWindow.connect( 'unmap-event', self.on_main_hidden, status )
        self.mainWindow.hide()
        if not TrayIcon.disabled:
            self.trayIcon.tray.hide()
        
        try:
            self.mainWindow.saveToQuit()
        except:
            pass

        if self.connected:
            self.msn.logout()
            self.config.writeUserConfig()
        
        self.config.writeGlobalConfig()
        sys.exit( status )

    def getMediaEnabled( self ):
        '''Get if can set media'''
        return self.mediaEnabled
        
    def setMediaEnabled( self, enabled=True):
        '''Set if can set media'''
        self.mediaEnabled = enabled
        
        if enabled:
            self.config.user['mediaEnabled'] = True
        else:
            self.config.user['mediaEnabled'] = False 
            
        if not enabled:
            self.msn.changeCurrentMedia('')

    def checkPending( self ):
        '''Check for users pending to be added'''
        users = self.msn.checkPending()
        if len( users ) > 0:
            for i in users:
                self.userAddedYouDialog(i, i)
        return False

    def startPlugins( self, plugins ):
        '''Start the plugins in pluginList'''

        if plugins[ 0 ] != '':
            for i in plugins:
                try:
                    (success, message) = self.pluginManager.checkPlugin( i )
                except:
                    success = False
                    message = _( 'invalid check() return value' )
                if not success:
                    emesenelib.common.debug( _( 'plugin %s could not be initialized, reason:' )%i )
                    emesenelib.common.debug( message )
                else:
                    self.pluginManager.startPlugin( i )

    def changeAvatarDialog( self ):
        ''' Shows a dialog to change personal avatar '''
        response, filename = AvatarChooser(parent = self.mainWindow, config = self.config).run()
        if response == gtk.RESPONSE_OK:
            self.changeAvatar(filename)
        elif response == RESPONSE_NO_AVATAR:
            self.changeAvatar(False)

    def changeAvatar( self, filename ):
        if not filename or not os.path.exists( filename ):
            filename = ''

        self.avatar = Avatar.Avatar( filename, self.config.getAvatarsCachePath() )
        if filename == '':
            self.msn.setDisplayPicture( '' )
            self.mainWindow.setAvatar( gtk.gdk.pixbuf_new_from_file( PATH.DEFAULT_THEME_PATH + 'userPanel.png' ) )
            self.config.user['avatarPath'] = ''
        else:
            self.msn.setDisplayPicture( self.avatar.getImagePath() )
            self.mainWindow.setAvatar( self.avatar.getThumb() )
            self.config.user['avatarPath'] = self.avatar.getImagePath()
        
        self.emit('avatar-changed')

    def changeNickDialog( self, user = 'Me', errorOnEmpty=True ):
        '''show a dialog requesting the nick to be changed
        and change it.'''

        if not self.connected:
            return

        if user == 'Me':
            response, pm = Dialogs.ChangeNick( self.mainWindow , self.msn.nick, self.msn.personalMessage ).run()
        else:
            alias = self.msn.contactManager.getContactAlias( user )
            deleteBut = True
            if not alias:
                alias = ''
                deleteBut = False
            response, pm = Dialogs.ChangeNick(self.mainWindow, alias, self.msn.personalMessage, \
                deleteBut).run()

        if response == None:
            return
        elif errorOnEmpty and response == '':
            Dialogs.Error( self.mainWindow , _( 'Empty Nickname' ) ).run()
            return

        self.changeNick( response, user )
        self.changePersonalMessage(pm)

    def changeNick( self , nick, user = 'Me' ):
        '''chang the nick in the session'''
        self.msn.changeNick( nick, user )

    def changePersonalMessage( self, pm ):
        '''change your personal message'''
        self.msn.changePersonalMessage( pm )

    def setStatus( self , status, source='' ):
        '''set the status'''

        if ( self.mainWindow.currentInterface == 'userlist' ):
            self.msn.changeStatus( status )
            if not TrayIcon.disabled:
                self.trayIcon.update(self.msn.status)

    def addUserDialog( self, toGroup=None ):
        '''show a dialog requesting the email of the ser to be added'''
        if not self.connected:
            return

        response = Dialogs.AddFriend( self.mainWindow, self.msn.getGroupNames(), toGroup ).run()

        if response == None:
            return
        elif response[ 0 ] == '':
            Dialogs.Error( self.mainWindow , _( 'Empty Field' ) ).run()
            return

        self.addUser( response[ 0 ], response[ 1 ] )
        self.refreshUserList()

    def addUser( self, email, group = '' ):
        '''add an user'''
        self.msn.addUser( email, group )

    def removeUser( self, email = None ):
        '''remove an user, if email is None the email will be the one selected
        on the user list'''

        if email == None:
            try:
                email = self.mainWindow.userList.getSelected()[ 1 ].email
            except:
                return

        if email == '':
            Dialogs.Error( self.mainWindow , _( 'Choose an user to remove' ) ).run()
            return

        if Dialogs.Confirm( self.mainWindow , _( 'Are you sure you want to remove %s from your contact list?' )%email, \
                        gtk.STOCK_DELETE ).run() == gtk.RESPONSE_ACCEPT:
            self.msn.removeUser( email )

        self.refreshUserList()

    def blockUser( self, email = None ):
        '''block an user, if email is None the email will be the one selected
        on the user list'''

        if email == None:
            try:
                email = self.mainWindow.userList.getSelected()[ 1 ].email
            except:
                return

            if email == '':
                Dialogs.Error( self.mainWindow , _( 'Choose an user' ) ).run()
                return

            self.msn.blockUser( email )

        else:
            self.msn.blockUser( email )
        
        self.refreshUserList()

    def unblockUser( self, email = None ):
        '''unblock an user, if email is None the email will be the one selected
        on the user list'''

        if email is None:
            try:
                email = self.mainWindow.userList.getSelected()[ 1 ].email
            except:
                return

            if not email:
                Dialogs.Error( self.mainWindow , _( 'Choose an user' ) ).run()
                return

        self.msn.unblockUser( email )

        self.refreshUserList()

    def addGroupDialog( self ):
        '''show a dialog requesting the name of the group to be added'''

        if not self.connected:
            return

        response = Dialogs.AddGroup( self.mainWindow ).run()

        if response == None:
            return
        elif response == '':
            Dialogs.Error( self.mainWindow , _( 'Empty Field' ) ).run()
            return

        self.addGroup( response )
        self.refreshUserList()

    def addGroup( self, group ):
        '''add a group'''

        self.msn.addGroup( group )

    def addNotification( self, msnp, command, tid, params, email, nick ):
        '''this method is called when a user adds you'''

        self.userAddedYouDialog(nick, email)

    def userAddedYouDialog( self, nick, email ):
        d = Dialogs.AddBuddy( self.mainWindow , nick + ' (' + email + ')'  )
        response = d.run()

        if response == gtk.RESPONSE_ACCEPT:
            self.addUser(email)
        else:
            pass # TODO make the user that adds you not able to see you!!

    def removeNotification( self, msn, command, tid, params, mail ):
        '''this method is called when a user removes you'''

        #Dialogs.Information( self.mainWindow , _( 'The user ' ) + mail + _( ' has deleted you' ) ).run()
        pass

    def removeGroup( self, group = None ):
        '''remove a group, if group is None the group will be the selected on the userlist'''

        if group == None:
            try:
                group = self.mainWindow.userList.getGroupSelected().name
            except:
                return

        if group == '':
            Dialogs.Error( self.mainWindow , _( 'Choose a group to delete' ) ).run()
            return

        if Dialogs.Confirm( self.mainWindow , _( 'Are you sure you want to delete %s?' )%group, \
            gtk.STOCK_DELETE ).run() == gtk.RESPONSE_ACCEPT:
            self.msn.removeGroup( group )

        self.refreshUserList()

    def renameGroupDialog( self, group = None ):
        '''show a dialog requesting the new name of the group, if group is None
        use the selected group in the userlist'''

        if group == None:
            try:
                group = self.mainWindow.userList.getGroupSelected().name
            except:
                pass

            if group == '':
                Dialogs.Error( self.mainWindow , _( 'Choose a group' ) ).run()
                return

        response = Dialogs.RenameGroup( self.mainWindow , group ).run()

        if response == None:
            return
        elif response == '':
            Dialogs.Error( self.mainWindow , _( 'Empty group name' ) ).run()
            return

        self.renameGroup( group, response )
        self.refreshUserList()

    def renameGroup( self, oldName, newName ):
        '''rename a group from oldName to newName'''

        self.msn.renameGroup(oldName, newName)

    def preferencesDialog( self ):
        '''show a preference dialog'''
        if not self.prefsOpen:
            p = PreferenceWindow.PreferenceWindow( self, self.config, self.mainWindow )
            p.show()
            self.prefsOpen = True

    def themeChanged( self ):
        '''called when the theme changes'''

        theme = self.config.user['theme']

        isSmall = self.config.user['smallIcons']
        smilieTheme = self.config.user['smilieTheme']
        
        error = False
        if not self.theme.setTheme( theme, isSmall ):
            error = True
        if not self.theme.smilies.setTheme( smilieTheme ):
            error = True
        
        if error:
            Dialogs.Error( self.mainWindow, \
                _( 'Error loading theme, falling back to default' ) ).run()
            self.config.user['theme'] = 'default'
            self.config.user['smilieTheme'] = 'default'
            theme = 'default'
            stheme = 'default'

        self.mainWindow.rebuild()
        if self.trayStatusMenu:
            self.trayStatusMenu.set_submenu( StatusMenu.StatusMenu( self ) )
            self.trayIcon.menu.show_all()
            self.trayIcon.update( self.msn.status )
            
        return theme

    def setAutoReplyDialog( self ):
        '''show a dialog requesting the autoreply message'''

        if not self.connected:
            return

        response = Dialogs.SetAutoReply( self.mainWindow , self.autoReplyMessage ).run()

        if response == None:
            return
        elif response == '':
            Dialogs.Error( self , _( 'Empty AutoReply' ) ).run()
        else:
            self.autoReplyMessage = response
            self.config.user['autoReplyMessage'] = response

    def showUserData( self, data ):
        '''show the user nick or mail in the status bar'''
        self.mainWindow.showUserData( data )

    def refreshUserList( self, msnp=None, force=False ):
        '''call to refreshUserList on MainWindow that calls refresh on userList
        userList get the values of showOffline etc from config, so if you want
        to change something, change it on config.
        if it's loading contacts (ILN), don't do anything'''
        
        if self.lastILN == 0 or time.time() - self.lastILN > 1:
            self.mainWindow.refreshUserList(force)
            self.lastILN = 0

    # signal handlers
    def connectionProblem( self, msnp ):
        '''called when a PNG is not answered'''
        
        self.logout( False, True )
    
    def onInitialStatusChange( self, msnp, command, tid, params):
        '''callback for ILN messages. saves the timestamp to speed up login'''
        self.lastILN = time.time()
        
    def onStatusChange( self, msnp, command, tid, params ):
        '''callback for the status change event'''
        self.refreshUserList()

    def on_tray_disconnect( self, *args ):
        self.logout()

    def msnobjChanged( self, msnp, msnobj, wasOffline ):
        '''callback called when the user change his msnobj'''
        #if msnobj != None and not wasOffline:
        if msnobj != None:
            if msnobj.getCreator() == '':
                return

            self.pendingAvatars[msnobj.getCreator()] = msnobj

            if not self.idleTimeout:
                self.idleTimeout = gobject.timeout_add( 5000, \
                    self.idleadd, priority=gobject.PRIORITY_LOW )

    def idle( self ):
        '''called when the gobject mainloop is idle
        gobjects thinks that it means "all the time" so we add a
        five-second non-blocking delay'''

        stop = True

        if len(self.pendingAvatars.keys()) > 0 and self.msn:
            # pop a pending avatar
            creator = self.pendingAvatars.keys().pop(0)
            msnobj = self.pendingAvatars[creator]
            contact = self.getContact(creator)
            # get a switchboard connection
            sb = self.msn.getSwitchboard(creator)
            if contact and not self.theme.hasUserDisplayPicture(contact):
                # send p2p invite, etc
                sb.getDisplayPicture(creator)
            elif contact:
                self.msn.emit("display-picture-changed", sb, msnobj, 
                                creator)
            
            # delete from pending
            del self.pendingAvatars[creator]

            # if there are no more avatars left, stop
            stop = len(self.pendingAvatars.keys()) == 0
        
        if not stop:
            # wait 5 seconds without blocking
            self.idleTimeout = gobject.timeout_add( 20000, \
                self.idleadd, priority=gobject.PRIORITY_LOW )
        return False
        
    def idleadd( self, *args ):
        '''called 5 seconds after last idle call'''
        self.idleTimeout = 0
        gobject.idle_add( self.idle, priority=gobject.PRIORITY_LOW )
        return False

    def newConversation( self, msnp, mail, switchboard=None, weStarted=False ):
        '''This method is called when the user want to initiate a new
        conversation with someone, or when a switchboard has been created
        and we want to give a window to it.
        (callback for 'new conversation' signal emmited by msnp)
        weStarted is a boolean that indicate if we started the conversation or 
        a friend'''

        return self.conversationManager.newConversation( self.msn, mail, \
            switchboard, weStarted )

    def pluginDialog( self ):
        '''Show the pluginManagerDialog'''

        p = PluginManagerDialog.PluginManagerDialog( self.pluginManager, \
            self.mainWindow, self.config )
        p.show()

    def seeSpace( self, email ):
        '''gets the space url and opens it in a browser'''
        #self.msn.getSpaceData( email )
        #desktop.open( 'http://spaces.live.com/' + user )
        self.msn.getUserProfile( email )
        # FIXME: get space url

    def seeProfile( self, email ):
        '''opens the profile url in a browser'''
        desktop.open('http://members.msn.com/' + email )

    def offlineMessageWaiting( self, msnp, msnOIM ):
        '''process the OIM messages'''
        for i in msnOIM.getMessages():
            msnOIM.retrieve( i[ 'I' ] )

    def offlineMessageReceived( self, msnp, oim ):
        '''process the OIM message'''
        user, date, message = oim
        window, conv = self.newConversation( msnp, user['addr'], None, False )
        conv.receiveOIM(user['name'], message, date)
       
    def messageError( self, msnp, to, message, error ):
        window, conv = self.newConversation( self.msn, to, None, True )
        conv.receiveError(msnp, to, message, error)

    def userDisconnected( self, msnp, tid, params ):
        '''method called when the server disconnect us'''

        Dialogs.Error( self.mainWindow, \
            _( 'The server has disconnected you.' ) ).run()
        self.logout( False, False )

    def getGroupNames( self, ordered=True):
        '''Returns a list with the group names in the contact list'''
        
        groups = self.msn.getGroupNames()
        groups.sort( key= lambda x: x.lower() )
        
        return groups
    
    def getGroupName( self, id ):
        '''returns the name of the group with given id'''
        return self.msn.contactManager.getGroupName( id )
    
    def getGroupId( self, groupName ):
        '''Returns the id of a group'''
        return self.msn.contactManager.getGroupId( groupName )

    def moveUserToGroup( self, user, oldGroup, group ):
        '''move a user to a group'''

        self.msn.moveUserToGroup( user, oldGroup, group )

    def addUserToGroup( self, user, group ):
        '''add a user to a group'''

        if group == 'No group':
            return

        self.msn.addUserToGroup( user, group )

    def removeUserFromGroup( self, user, group ):
        '''remove a user from a group'''
        self.msn.removeUserFromGroup( user, group )
        
    def getMenuData( self ):
        return self.mainWindow.userList.getMenuData()
    
    # TODO: getters
    def getAvatar(self):
        return self.avatar
    
    def getConfig(self):
        return self.config
        
    def getTheme(self):
        return self.theme
        
    def getUnifiedParser( self ):
        return self.unifiedParser
        
    def getConversationLayoutManager(self):
        return self.conversationLayoutManager
        
    def getMSN(self):
        return self.msn
        
    def getNick( self ):
        if self.msn:
            return self.msn.nick
        else:
            return ''
        
    def getPersonalMessage( self ):
        if self.msn:
            return self.msn.personalMessage
        else:
            return ''
        
    def getUnreadMails( self ):
        if self.msn:
            return self.msn.inboxUnreadMessages
        else:
            return 0
            
    def getContactStatus( self, contact = None ):
        '''return the contact status, if the contact is None return
        the user status'''

        if not self.msn:
            return 'FLN'
        
        if contact:
            return self.msn.contactManager.getContactStatus(contact)
        else:
            return self.msn.status

    def getContact( self, mail ):
        '''return a contact object'''
        if not self.msn:
            return None

        return self.msn.contactManager.getContact( mail )

    def getContactAlias( self, contact = None ):
        '''return the contact alias'''

        if not self.msn:
            return ''

        return self.msn.contactManager.getContactAlias( contact )
        
    def getContactIsBlocked( self, contact = None ):
        if not self.msn:
            return False

        return self.msn.contactManager.getContactIsBlocked( contact )

    def updateDebug( self, *args ):
        if self.msn:
            self.msn.setDebug(self.config.glob['debug'], \
                self.config.glob['binary'])
        
        
loginStatus = 0
loginError = ''

def login( msn ):
    global loginStatus
    global loginError
    try:
        msn.login()
    except Exception, e:
        traceback.print_exc()
        loginStatus = -1
        print e
        loginError = e
        return False
    except:
        loginStatus = -1
        return False

    loginStatus = 1
    return False

def parseArgs():
    '''parses sys.argv with getopt
    returns tuple with Controller args'''
    username = ''
    minimized = False
    safe = False
    leakdebug = False
    
    # parse arguments
    try:
        import getopt
    except ImportError:
        #getopt is disabled in emesene py2exe
        return ('', False, False, False)
    
    shortArgs = 'm'
    longArgs = ['minimized', 'user=', 'safe', 'leakdebug']
    try:
        args = getopt.getopt(sys.argv[1:], shortArgs, longArgs)
    except getopt.GetoptError, e:
        # wrong input, display usage
        print e
        print 'Usage:', sys.argv[0], '[-m|--minimized] [--user=mail@address]'
        print 'Advanced options: --safe, --leakdebug'
        sys.exit(1)
        
    # parse getopt output
    for key, value in args[0]:
        if key == '--user':
            username = value
        if key == '-m' or key == '--minimized':
            minimized = True
        if key == '--safe':
            safe = True
        if key == '--leakdebug':
            leakdebug = True

    return (username, minimized, safe, leakdebug)

def main():
    try:
        path = os.path.dirname(__file__) or sys.path[0]
    except NameError:
        path = sys.path[0]

    originalPath = os.getcwd()
    os.chdir( path )
    sys.path.append(path)
    # start controller
    controller = Controller(*parseArgs())

    # this stuff is to interrupt blocking stuff safely.
    for i in range(5):
        try:
            gtk.main()
        except KeyboardInterrupt:
            print 'Interrupt (%d more times to close)' % i - 1
        else:
            break
    controller.quit(0)
    os.chdir( originalPath )

if __name__ == '__main__':
    main()
