# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


import gtk

class GroupMenu( gtk.Menu ):
	'''This class represent the popup menu that is displayed when you right click a group on
	the userList'''

	def __init__( self , controller, group ):

		gtk.Menu.__init__( self )

		self.controller = controller
		
		renameGroupMenuItem = self.newImageMenuItem( _( 'Re_name group...' ), \
													 gtk.STOCK_EDIT)
		self.add(renameGroupMenuItem)
		deleteGroupMenuItem = self.newImageMenuItem( _( 'Re_move group' ), \
													 gtk.STOCK_DELETE)
		self.add(deleteGroupMenuItem)
		
		self.add(gtk.SeparatorMenuItem())

		addUserGroupMenuItem = self.newImageMenuItem( _( '_Add contact...' ) , \
													  gtk.STOCK_ADD)
		self.add(addUserGroupMenuItem)
		addGroupMenuItem = self.newImageMenuItem( _( 'Add _group...' ), \
												  gtk.STOCK_ADD)
		self.add(addGroupMenuItem)

		self.show_all()

		addUserGroupMenuItem.connect('activate' , self.on_add_user_group_activate)
		addGroupMenuItem.connect('activate' , self.on_add_group_activate)
		deleteGroupMenuItem.connect('activate' , self.on_delete_group_activate)
		renameGroupMenuItem.connect('activate' , self.on_rename_group_activate)

		self.group = group

	def newImageMenuItem( self , label , stock = None , img = None ):
		mi = gtk.ImageMenuItem(_(label))

		if stock != None:
			mi.set_image(gtk.image_new_from_stock(stock, gtk.ICON_SIZE_MENU))
		elif img != None:
			image = gtk.Image()
			image.set_from_pixbuf( img )
			mi.set_image( image )
		return mi

	def on_add_user_group_activate( self , *args ):
		self.controller.addUserDialog( self.group.name )

	def on_add_group_activate( self , *args ):
		self.controller.addGroupDialog()

	def on_delete_group_activate( self , *args ):
		self.controller.removeGroup()

	def on_rename_group_activate( self , *args ):
		self.controller.renameGroupDialog( self.group.name )

