# -*- coding: utf-8 -*-
#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

# based on matz code
# icons stolen from gimp
# TODO: enhance drawing (better brushes, antialiasing)
#       check why the pixmap is cleared when resizing
#       simpler color picker, with popup menu
#       icons in sizesMenu
#       embed into ConversationUI
#       pseudo tabs to switch in conversation statusbar
#       the lack of docstrings hurts my pylint instinct
#       convert pixbufs to gif with pygif

import Theme
import gtk

# gettext
_ = lambda x:x

class InkDraw( gtk.Window ):
    def __init__(self, theme):
        gtk.Window.__init__(self)
        self.theme = theme

        self.vbox = gtk.VBox()
        self.hbox = gtk.HBox()

        self.color = gtk.gdk.Color(150, 150, 150)
        self.size = 6 
        self.pixmap = None
        self.paint = True

        ### Drawing Area
        self.draw_area = gtk.DrawingArea()
        size = 200, 200
        self.draw_area.set_size_request(*size)
        self.draw_area.show()
        self.draw_area.connect("expose_event", self.expose_event)
        self.draw_area.connect("configure_event", self.configure_event)
        self.draw_area.connect("motion_notify_event", self.motion_notify_event)
        self.draw_area.connect("button_press_event", self.button_press_event)
        self.draw_area.connect("realize", self.realize)
        self.draw_area.set_events(gtk.gdk.EXPOSURE_MASK
                                    | gtk.gdk.LEAVE_NOTIFY_MASK
                                    | gtk.gdk.BUTTON_PRESS_MASK
                                    | gtk.gdk.POINTER_MOTION_MASK
                                    | gtk.gdk.POINTER_MOTION_HINT_MASK)

        ### Color select
        self.pixelColor = gtk.ToolButton()
        self.pixelColor.set_stock_id(gtk.STOCK_SELECT_COLOR)
        self.pixelColor.connect("clicked", self.clickColor)
        
        ### paint button
        self.paintImage = gtk.Image()
        self.paintImage.set_from_pixbuf(self.theme.getImage('paintbrush'))
        self.paintButton = gtk.MenuToolButton(self.paintImage, _('Paint') )
        self.paintButton.set_menu(sizesMenu(self.paintSize))
        self.paintButton.connect('clicked', self.setPaint)
        
        ### eraser button
        self.eraseImage = gtk.Image()
        self.eraseImage.set_from_pixbuf(self.theme.getImage('eraser'))
        self.eraseButton = gtk.MenuToolButton(self.eraseImage, _('Eraser') )
        self.eraseButton.set_menu(sizesMenu(self.eraserSize))
        self.eraseButton.connect('clicked', self.setEraser)

        ### clear button
        imgclear = gtk.Image()
        imgclear.set_from_stock(gtk.STOCK_CLEAR, gtk.ICON_SIZE_LARGE_TOOLBAR )
        self.clear = gtk.ToolButton( imgclear, _('Clear') )
        self.clear.connect( 'clicked', self.clearImage )

        ### hbox
        self.hbox.pack_start(self.pixelColor, False, False)
        self.hbox.pack_start(self.paintButton, False, False)
        self.hbox.pack_start(self.eraseButton, False, False)
        self.hbox.pack_start(self.clear, False, False)
        ### vbox
        self.vbox.pack_start(self.draw_area)
        self.vbox.pack_start(self.hbox)
        self.add(self.vbox)
        self.show_all()
    
    def setPaint(self, widget=None):
        self.paint = True

    def setEraser(self, widget=None):
        self.paint = False

    def paintSize(self, widget, order):
        self.setSize(order)
        self.setPaint()

    def eraserSize(self, widget, order):
        self.setSize(order)
        self.setEraser()
        
    def setSize(self, order):
        self.size = order * 5 + 1

    def clickColor(self,widget):
        colorDialog = gtk.ColorSelectionDialog( _( 'Choose a color' ) )
        colorDialog.colorsel.set_has_palette( True )
        response = colorDialog.run()
        if response == gtk.RESPONSE_OK:
            self.color = colorDialog.colorsel.get_current_color()
        colorDialog.destroy()

    def expose_event( self , widget , event):
        x , y, width, height = event.area
        self.draw_area.window.draw_drawable( \
            self.draw_area.get_style().fg_gc[gtk.STATE_NORMAL],     
            self.pixmap, x, y, x, y, width, height)
        return False
    
    def realize( self, widget):
        x, y, width, height = self.draw_area.get_allocation()
        self.pixmap = gtk.gdk.Pixmap(self.draw_area.window, width, height)
        self.pixmap.draw_rectangle(self.draw_area.get_style().white_gc,True, 0, 0, width, height)
        return True

    def configure_event( self, widget, event):
        x, y, width, height = self.draw_area.get_allocation()
        self.pixmap = gtk.gdk.Pixmap(self.draw_area.window, width, height)
        self.pixmap.draw_rectangle(self.draw_area.get_style().white_gc,True, 0, 0, width, height)
        return True


    def button_press_event( self, widget, event):
        if event.button == 1 and self.pixmap != None:
            self.draw_brush(event.x,event.y)
        return True

    def motion_notify_event( self, widget, event):
        if event.is_hint:
            x, y, state = event.window.get_pointer()
        else:
            x = event.x
            y = event.y
            state = event.state

        if state & gtk.gdk.BUTTON1_MASK and self.pixmap != None:
            self.draw_brush(x, y)
        return True

    def clearImage(self,widget):
         x, y, width, height = self.draw_area.get_allocation()
         self.pixmap.draw_rectangle(self.draw_area.get_style().white_gc, \
            True, 0, 0, width, height)
         self.draw_area.queue_draw_area(0, 0, width, height)
    
    def draw_brush(self, x, y):
        # @alen: refactor me!
        off = int ( self.size / 2 )
        rect = (int(x-off), int(y-off), self.size,self.size)
        win =  self.draw_area.window
        context = win.new_gc()
        cmap = self.draw_area.get_colormap()
        if self.paint:
            color =  cmap.alloc_color(self.color)
            context.foreground = color
        else:
            context = self.draw_area.get_style().white_gc
        self.pixmap.draw_rectangle(context, True, *rect)
        self.draw_area.queue_draw_area(*rect)
    
    def get_pixbuf(self):
        size = self.pixmap.get_size()
        pixbuf = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB,0,8, *size)
        pixbuf = pixbuf.get_from_drawable(self.pixmap, \
            self.draw_area.get_colormap(), 0, 0, 0, 0, *size)
        return pixbuf
    
def sizesMenu(callback):
    '''builds a menu with brush sizes'''
    menu = gtk.Menu()
    
    sizes = (_('Small'), _('Medium'), _('Large'))
    order = 0
    for size in sizes:
        order += 1
        item = gtk.MenuItem(size)
        item.connect("activate", callback, order)
        item.show()
        menu.append(item)
    return menu


if __name__ == "__main__":
    theme = Theme.Theme(None)
    win = InkDraw(theme)
    win.connect('destroy', gtk.main_quit)
    gtk.main()
    pb = win.get_pixbuf()
    pb.save("ink_test.png", "png")
    
