# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import gtk
import Dialogs
from Theme import resizePixbuf
from gobject import timeout_add, source_remove
from ImageFileChooser import ImageFileChooserDialog

class SmilieWindow( gtk.Window ):
    '''this is the window that opens when you press the smilie button on the
    conversation window'''

    def __init__( self, controller, callback, parent=None ):

        gtk.Window.__init__( self )
        self.set_decorated( False )
        self.set_type_hint( gtk.gdk.WINDOW_TYPE_HINT_DIALOG )

        if parent:
            self.set_transient_for( parent )
        self.set_position( gtk.WIN_POS_MOUSE )
        self.set_resizable( False )
        self.set_title( _('Smilies') )
        self.set_border_width( 2 )

        self.callback = callback
        
        self.vbox = gtk.VBox()
        table1 = gtk.Table(10)
        
        self.theme = controller.getTheme()
        
        self.customEmoticons = controller.customEmoticons
        self.config = controller.config
        
        self.closed = True
        
        self.tooltips = gtk.Tooltips()
        
        x = 0
        y = 0

        for i in self.theme.getSingleSmileysList():
            if x == 10:
                x = 0
                y += 1
            try:
                button = gtk.Button()
                button.set_relief(gtk.RELIEF_NONE)
                self.tooltips.set_tip(button, i)
                smilieImage = gtk.Image()
                if self.theme.getSmiley( i ).is_static_image():
                    smilieImage.set_from_pixbuf( self.theme.getSmiley( i ).get_static_image() )
                else:
                    smilieImage.set_from_animation( self.theme.getSmiley( i ) )
                button.set_image( smilieImage )
                button.connect( 'clicked' , self.clicked , i )
                button.connect( 'enter-notify-event' , self.on_enter_notify_event )
                table1.attach( button , x , x+1 , y , y+1 )
            except:
                x -= 1
            x += 1

        ceLabel = gtk.Label('Custom Emoticons')
        self.vbox.pack_start( table1 )
        self.vbox.pack_start( ceLabel )
        self.fillCETable()
        
        self.add( self.vbox )
        self.vbox.show_all()
        self.connect( 'delete-event' , self.on_delete_event )
        self.connect( 'leave-notify-event' , self.on_leave_notify_event )
        self.connect( 'enter-notify-event' , self.on_enter_notify_event )

        self.tag = None

    def on_leave_notify_event(self, *args):
        if not self.tag and not self.closed:
            self.tag = timeout_add( 500, self.hide )

    def on_enter_notify_event(self, *args):
        if ( self.tag ):
            source_remove( self.tag )
            self.tag = None

    def on_delete_event( self , *args ):
        self.hide()
        self.closed = True
        return True
    
    def clicked( self, button, smilie ):
        self.hide()
        self.callback( smilie )

    def event( self, button, event, smilie ):
        if event.type == gtk.gdk.BUTTON_PRESS and event.button == 1:
            self.hide()
            self.callback( smilie )
        if event.type == gtk.gdk.BUTTON_PRESS and event.button == 3:
        
            # -- Custom Emoticon Menu --
            self.emoMenu = gtk.Menu()
            
            self.shortcutItem = gtk.ImageMenuItem( _( "Change shortcut" ) )
            self.shortcutItem.set_image( gtk.image_new_from_stock(gtk.STOCK_EDIT, gtk.ICON_SIZE_MENU) )
            self.shortcutItem.connect( "activate", self.onEditShortcut, smilie )
            
            self.deleteItem = gtk.ImageMenuItem( _( "Delete" ) )
            self.deleteItem.set_image( gtk.image_new_from_stock(gtk.STOCK_DELETE, gtk.ICON_SIZE_MENU) )
            self.deleteItem.connect( "activate", self.onDeleteEmo, smilie )
            
            self.emoMenu.add( self.shortcutItem )
            self.emoMenu.add( self.deleteItem )
            
            self.emoMenu.show_all()
            # -----
            
            self.emoMenu.popup( None, None, None, event.button, event.time )
    
    def onDeleteEmo( self, button, shortcut ):
        self.customEmoticons.delete( shortcut )
        self.fillCETable()
    
    def onEditShortcut( self, button, shortcut ):
        self.hide()
        
        newShortcut = EditCEShortcut( self, shortcut, self.customEmoticons.list[ shortcut ] ).run()  
        
        if newShortcut != None:
            ret,msg = self.customEmoticons.chageShortcut( shortcut, newShortcut )
            if not ret:
                Dialogs.Error( None, msg ).run()
            
    def clickedAdd( self, button, *args ):
        self.hide()

    def show( self ):
        self.fillCETable()
        gtk.Window.show( self )
        self.tag = None
        self.closed = False

    def hide( self ):
        gtk.Window.hide( self )
        if ( self.tag ):
            source_remove( self.tag )
            self.tag = None
        self.closed = True
        
    def fillCETable( self ):
        try:
            self.vbox.remove( self.ceTable )
        except: pass
        self.ceTable = gtk.Table()
        x = 0
        y = 0
        
        for shortcut, filename in self.customEmoticons.list.iteritems():
            if x == 10:
                x = 0
                y += 1
            try:
                pixbuf = gtk.gdk.pixbuf_new_from_file( filename )
                pixbuf = resizePixbuf( pixbuf, 24, 24 )
                button = gtk.Button( )
                button.set_relief(gtk.RELIEF_NONE)
                self.tooltips.set_tip(button, shortcut)
                smilieImage = gtk.Image()
                smilieImage.set_from_pixbuf( pixbuf )
                button.set_image( smilieImage )
                button.connect( 'event' , self.event, shortcut )
                button.connect( 'enter-notify-event' , self.on_enter_notify_event )
                self.ceTable.attach( button, x , x+1 , y , y+1 )
            except:
                print 'Error in smiley', shortcut, filename
                x -= 1
            x += 1
            
        button = gtk.Button( None, gtk.STOCK_ADD )
        button.set_image( gtk.image_new_from_stock( gtk.STOCK_ADD, gtk.ICON_SIZE_BUTTON ) )
        button.connect( 'clicked', self.addClicked )
        button.connect( 'enter-notify-event' , self.on_enter_notify_event )
        self.ceTable.attach( button, x , x+1 , y , y+1 )
        
        self.vbox.pack_start( self.ceTable )
        self.vbox.show_all()
        
    def addClicked( self, button ):
        self.hide()
        CEImageChooser( self.customEmoticons, self.config ).run()

class EditCEShortcut( gtk.Dialog ):
    '''edit a custom emoticon shortcut'''
    
    def __init__( self , parent,  shortcut, image ):
        gtk.Dialog.__init__( self , _('Change Custom Emoticon shortcut'), parent , \
                gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT , \
                (gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT, \
                gtk.STOCK_OK, gtk.RESPONSE_ACCEPT) )
        
        self.set_resizable( False )
        self.set_has_separator( False )
        self.set_border_width( 6 )
        
        self.hbox = gtk.HBox()
        
        self.image = gtk.Image()
        self.image.set_from_file( image )
        
        shortcutLabel = gtk.Label( _( 'Shortcut: ' ) )
        self.shortcut = gtk.Entry()
        self.shortcut.set_text( shortcut )
        
        self.hbox.pack_start( self.image )
        self.hbox.pack_start( shortcutLabel )
        self.hbox.pack_start( self.shortcut )
        
        self.vbox.pack_start( self.hbox )
        self.vbox.show_all()
    
    def run( self ):
        '''runs the dialog and return the new shortcut'''
        
        option = gtk.Dialog.run( self )
        if option == gtk.RESPONSE_ACCEPT:
            shortcut = self.shortcut.get_text().replace(' ', '')
            
            self.destroy()
            return shortcut
        else:
            self.destroy()
            return None

class CEImageChooser( gtk.FileChooserDialog ):
    
    def __init__( self, customEmoticons, config ):
    
        self.config = config
        self.customEmoticons = customEmoticons
        
        title = ( "Add emoticon" )
    
        gtk.FileChooserDialog.__init__( self, title, None, \
                                gtk.FILE_CHOOSER_ACTION_OPEN, \
                                ( gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, \
                                  gtk.STOCK_OPEN, gtk.RESPONSE_OK ) )
        
        self.set_default_response( gtk.RESPONSE_OK )
        
        self.addFilter()
        
        self.lastFolder = self.config.user['emoticonDir']
        
        self.image = gtk.Image()
        self.image.show()
        self.set_preview_widget ( self.image )
        
        self.shortcutEntry = gtk.Entry()
        
        self.shortcutLabel = gtk.Label( _( 'Shortcut:' ) ) 
        self.comboLabel = gtk.Label( _( 'Size:' ) )
        
        self.combobox = gtk.combo_box_new_text()
        self.combobox.append_text( _( 'Small' ) + " (19x19)")
        self.combobox.append_text( _( 'Large' ) + " (50x50)")
        self.combobox.set_active(0)

        self.set_preview_widget ( self.image )
        self.connect( 'update-preview', self.onUpdate )
        

        if self.lastFolder or not self.lastFolder == '':
            self.set_current_folder( self.lastFolder )
            
        self.hbox = gtk.HBox()
        
        self.hbox.pack_start( self.shortcutLabel )
        self.hbox.pack_start( self.shortcutEntry )
        self.hbox.pack_start( self.combobox )
        
        self.vbox.pack_start( self.hbox , False, False )
        
        self.show_all()
    
    def run( self ):
        while True: # break and continue
            response = gtk.FileChooserDialog.run( self )
            
            if response != gtk.RESPONSE_OK:
                # cancel
                break
                
            self.config.user['emoticonDir'] = self.get_current_folder()
            
            CEPath = self.get_filename()
            Shortcut = self.shortcutEntry.get_text()
            Size = self.combobox.get_active()
            
            if CEPath == None:
                # nothing selected
                continue

            ret,msg =  self.customEmoticons.create( Shortcut , CEPath, Size )
            if ret:
                break
            else:
                Dialogs.Error( None, msg ).run()
                continue

        self.hide()
    
    def onUpdate( self, fileChooser  ):
        self.image.hide()        
        self.set_preview_widget_active( False )
        try:
            pixbuf = gtk.gdk.pixbuf_new_from_file( self.get_filename() )
            
            if pixbuf.get_width() > 128 and pixbuf.get_height() > 128:
                pixbuf = pixbuf.scale_simple( 128, 128, gtk.gdk.INTERP_BILINEAR )
            
            self.image.set_from_pixbuf( pixbuf )
            
            self.set_preview_widget_active( True )
            self.image.show()
            
            size =  self.get_size()
            width = size[0] 
            height = size[1]
            self.resize( width, height )
            
        except:
            pass
            
    def addFilter( self ):
        '''
        Adds all the possible file filters to the dialog. The filters correspond
        to the gdk available image formats
        '''
        
        # All images filter
        allImages = gtk.FileFilter()
        allImages.set_name( _( 'All images' ) )
        
        filters = []
        formats = gtk.gdk.pixbuf_get_formats()
        for format in formats:
            filter = gtk.FileFilter()
            name = "%s (*.%s)" % ( format['description'], format['name'] )
            filter.set_name( name )
            
            for mtype in format['mime_types']:
                filter.add_mime_type( mtype )
                allImages.add_mime_type( mtype )
                
            for pattern in format['extensions']:
                tmp = '*.' + pattern
                filter.add_pattern( tmp )
                allImages.add_pattern( tmp )
                
            filters.append( filter )

        self.add_filter( allImages )
        self.set_filter( allImages )
        for filter in filters:
            self.add_filter( filter )
