# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import gtk
import re
import desktop
from Parser import Url

class UserMenu( gtk.Menu ):
    '''This class represents the popup menu that is displayed when you right
    click an user on the userList'''

    def __init__( self , controller, user, group ):
        gtk.Menu.__init__( self )

        self.controller = controller
        self.contactManager = self.controller.getContactManager()

        openConversationMenuItem = self.newImageMenuItem( _( "_Open Conversation" ), gtk.STOCK_OPEN )
        self.add( openConversationMenuItem )
        openConversationMenuItem.show()

        renameUserMenuItem = self.newImageMenuItem( _( "_Set contact alias..." ), gtk.STOCK_EDIT )
        self.add( renameUserMenuItem )
        renameUserMenuItem.show()
        
        profileMenuItem = self.newImageMenuItem( _( "View profile" ), gtk.STOCK_NETWORK )
        self.add( profileMenuItem )
        profileMenuItem.show()
        profileMenuItem.connect( 'activate', self.on_profile_activate )
        
        parser = controller.getUnifiedParser()
        sep = False

        for i in user.nick,user.personalMessage:

            urls = [str(x.url) for x in parser.parse(None, i) if type(x) == Url]
            for url in urls:

                if len(url) > 20:
                    dispUrl = url[:20] + '...'
                else:
                    dispUrl = url
            
                PMurl = self.newImageMenuItem(dispUrl, gtk.STOCK_OPEN)
                PMurl.connect("activate", self.on_PMurl, url) 
            
                if not sep:
                    self.add( gtk.SeparatorMenuItem() )
                    sep = True
                self.add(PMurl)
        
        if sep:
            self.add( gtk.SeparatorMenuItem() )

        copyEmail = self.newImageMenuItem( _("Copy email"), gtk.STOCK_COPY)
        self.add( copyEmail )
        copyEmail.show()
            
        deleteAlias = gtk.MenuItem( _( '_Delete alias' ) )
        deleteAlias.set_sensitive( not user.alias == '')
        deleteAlias.connect( 'activate', self.on_delete_alias_activate )
        self.add( deleteAlias )
        
        self.add( gtk.SeparatorMenuItem() )

        if not user.blocked:
            blockUserMenuItem = self.newImageMenuItem( _( "_Block" ), gtk.STOCK_STOP )
            self.add( blockUserMenuItem )
            blockUserMenuItem.show()
            blockUserMenuItem.connect( "activate" , self.on_block_user_activate )
        else:
            unblockUserMenuItem = self.newImageMenuItem( _( "_Unblock" ) )
            self.add( unblockUserMenuItem )
            unblockUserMenuItem.show()
            unblockUserMenuItem.connect( "activate" , self.on_unblock_user_activate )

        # uncomment this when we find a better way to know the space URL
        #if self.contactManager.getContactHasSpace( userName ):
        #    self.add( gtk.SeparatorMenuItem() )
        #    spaceUserMenuItem = self.newImageMenuItem( _( "Visit MSN Space" ) , gtk.STOCK_NETWORK )
        #    self.add( spaceUserMenuItem )
        #    spaceUserMenuItem.show()
        #    spaceUserMenuItem.connect( "activate" , self.on_space_user_activate )

        deleteUserMenuItem = self.newImageMenuItem( _( "_Remove contact" ), gtk.STOCK_DELETE )
        self.add( deleteUserMenuItem )
        deleteUserMenuItem.show()

        self.add( gtk.SeparatorMenuItem() )

        moveMenuItem = self.newImageMenuItem( _( "M_ove to group" ), gtk.STOCK_REDO )
        moveMenu = gtk.Menu()

        for i in self.controller.getGroupNames():
            menuItem = self.newImageMenuItem ( i )
            moveMenu.add( menuItem )
            menuItem.connect( "activate" , self.moveToActivate, i )

        moveMenuItem.set_submenu( moveMenu )
        moveMenuItem.show_all()
        self.add( moveMenuItem )

        copyMenuItem = self.newImageMenuItem( _( "_Copy to group" ), gtk.STOCK_COPY )
        copyMenu = gtk.Menu()

        for i in self.controller.getGroupNames():
            menuItem = self.newImageMenuItem ( i )
            copyMenu.add( menuItem )
            menuItem.connect( "activate" , self.copyToActivate, i )

        copyMenuItem.set_submenu( copyMenu )
        copyMenuItem.show_all()
        self.add( copyMenuItem )

        removeFromGroupMenuItem = self.newImageMenuItem( _( "R_emove from group" ), gtk.STOCK_DELETE )
        self.add( removeFromGroupMenuItem )
        removeFromGroupMenuItem.show()
        removeFromGroupMenuItem.connect( "activate" , self.removeFromActivate )

        self.add( gtk.SeparatorMenuItem() )

        addUserMenuItem = self.newImageMenuItem( _( "_Add contact..." ) , gtk.STOCK_ADD )
        self.add( addUserMenuItem )
        addUserMenuItem.show()

        self.show_all()

        renameUserMenuItem.connect( "activate" , self.on_rename_user_activate )
        addUserMenuItem.connect( "activate" , self.on_add_user_activate )
        deleteUserMenuItem.connect( "activate" , self.on_delete_user_activate )
        openConversationMenuItem.connect( "activate" , self.on_open_conversation_activate )
        copyEmail.connect("activate", self.on_copy_email_activate)

        self.user = user
        self.group = group

    def newImageMenuItem( self , label , stock = None , img = None ):
        mi = gtk.ImageMenuItem( _( label ) )

        if stock != None:
            mi.set_image(gtk.image_new_from_stock(stock, gtk.ICON_SIZE_MENU))
        elif img != None:
            image = gtk.Image()
            image.set_from_pixbuf( img )
            mi.set_image( image )
        return mi

    def on_PMurl( self , widget , url):
        desktop.open(str(url))

    def moveToActivate( self, menuItem, group ):
        self.controller.moveUserToGroup( self.user.email, self.group.name, group )

    def copyToActivate( self, menuItem, group ):
        self.controller.addUserToGroup( self.user.email, group )

    def removeFromActivate( self, menuItem ):
        self.controller.removeUserFromGroup( self.user.email, self.group.name )
        
    def on_delete_alias_activate( self, *args ):
        self.controller.changeNick( '', self.user.email )

    def on_add_user_activate( self , *args ):
        self.controller.addUserDialog()

    def on_delete_user_activate( self , *args ):
        self.controller.removeUser( self.user.email )

    def on_rename_user_activate( self , *args ):
        self.controller.changeNickDialog( self.user.email, False )

    def on_block_user_activate( self , *args ):
        self.controller.blockUser( self.user.email )

    def on_unblock_user_activate( self , *args ):
        self.controller.unblockUser( self.user.email )

    def on_space_user_activate( self , *args ):
        self.controller.seeSpace( self.user.email )

    def on_profile_activate( self , *args ):
        self.controller.seeProfile( self.user.email )

    def on_open_conversation_activate( self, *args ):
        self.controller.newConversation( None, self.user.email, None, True )
    
    def on_copy_email_activate( self, *args ):
        clipboard = gtk.clipboard_get(gtk.gdk.SELECTION_CLIPBOARD)
        clipboard.set_text(self.user.email)

