# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import os
import time
import uuid
import base64
import socket
import urllib
import random
import gobject

import Msnobj
import Socket # emesenelib
import msn_p2p

import common

class Switchboard( gobject.GObject ):
    '''This class represent a switchboard connection with the notification server.'''
    
    __gsignals__ = {
        'typing' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,)),

        # body, format, charset
        'message-sent' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_PYOBJECT, gobject.TYPE_PYOBJECT, 
                gobject.TYPE_PYOBJECT)),
        'nudge-sent' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            ()),
        'ink-sent' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            ()),
        # data
        'action-sent' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,)),
        'user-join' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,)),
        'user-leave' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,)),
        'user-offline' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,)),
        'user-online' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,)),

        'message' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            #tid, nick, body, format, charset
            (gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING, 
             gobject.TYPE_STRING, gobject.TYPE_STRING)),
        'action-message' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING, gobject.TYPE_STRING)),
            #tid, data
        'ink-message' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING, gobject.TYPE_STRING)),
            #tid, filename

        'nudge' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,)),
        'status-change' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
            
        # this signal is emited when a message with a custom emoticon is received
        'custom-emoticon-received' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,gobject.TYPE_PYOBJECT,)),
        
    } 

    def __init__( self , id, msn, type, status = 'pending' ):
        '''class contructor
        type can be 'requested' or 'invited', its 'requested' if we request the
        switchboard and guess what invited means? :P.
        status is pending initialy, then when we process the connection string is
        connected and is stablished when someone join the conversation, 
        if something goes wrong the status is closed'''

        gobject.GObject.__init__(self)
        self.id = id
        
        self.status = status
        self.validStatus = [ 'pending', 'connected', 'established', 'closed', 'error' ]
        self.invalidTransitions = { 'connected' : [ 'pending' ], 'established' : [ 'pending', 'connected' ] }
        self.connectionString = ''
        
        self.msn = msn

        self.user = self.msn.user.lower()
        self.proxy = self.msn.proxy
        
        self.host = ''
        self.port = 0
        self.authenticationType = ''
        self.authenticationString = ''
        self.sessionID = ''
        self.command = ''
        self.socket = None

        self.members = {}

        # messageQueue: a list that holds the messages sent when 
        # the status was 'pending'. It's a list of dicts like this
        # {'msg': msg, 'header': header, 'acknowledgeType': type}'
        self.messageQueue = [] 

        # invitationQueue: a list of mails to invite when we go
        # to established status
        self.invitationQueue = []
        # inkSessions: {session: dict{'file','total','actual'}}
        self.inkSessions = {}
        # pendingCEs {mail: [(creator, msnobj, name), ...]}
        # these are params to getCustomEmoticon()
        self.pendingCEs = {}
        
        self.userOfflineId = self.msn.connect('user-offline', self.userOffline)
        self.userOnlineId = self.msn.connect('user-online', self.userOnline)
        self.signals = []

        self.firstMessage = False
        self.firstUser = ''
        
        self.error = 0
        self.started = time.time()
   
    def __repr__( self ):
        return '<Switchboard users: ' + str( self.members.keys() ) + '>' 
   
    def parse( self ):
        '''parse the connection string to fill the attributes'''
    
        if self.connectionString == '':
            return        

        if self.connectionString.split()[ 0 ] == 'XFR':
            # example
            # XFR 15 SB 207.46.108.37:1863 CKI 17262740.1050826919.32308\r\n
            ( command, tid, sb, host, authenticationType, authenticationString, u, server) = self.connectionString.split()
        else:
            # example
            # RNG 11752013 207.46.108.38:1863 CKI 849102291.520491113 example@passport.com Example%20Name
            ( command, sessionID, host, authenticationType, authenticationString, user, userName, u, server ) = self.connectionString.split()
            self.sessionID = sessionID
             
            self.addMember( user, userName )

        self.command = command
        self.host = host.split( ':' )[ 0 ]
        self.port = int( host.split( ':' )[ 1 ] )
        self.authenticationType = authenticationType
        self.authenticationString = authenticationString
        
    def leaveChat( self ):
        '''leave the conversation'''
        
        self.setStatus( 'closed' )
        try:
            self.socket.sendText( 'OUT\r\n' )
        except Exception, e:
            pass

    def socketHangup( self, fd, condition ):
        '''The socket got into IO_HUP or IO_ERR status'''
        common.debug( str(self) + ' hangup/error', 'switchboard')
        self.leaveChat()
        return False
        
    def splitMsg( self, message ):
        '''return header(dict), body(str)'''
        part = message.split( '\r\n\r\n' )
        def htuple(x):
            parts = x.split(': ')
            return ( parts[0], ': '.join(parts[1:]) )
        header = dict([ htuple(i) for i in part[0].split('\r\n') ])
        body = '\r\n\r\n'.join(part[1:])
        return header,body
        
    def process( self, fd=None, condition=None ):
        '''read the info from Socket and process the info'''
        if self.socket == None:
            return False
        
        def close(message, status='closed'):
            common.debug(message, 'switchboard')
            self.setStatus(status)
            
        try:
            fds = self.socket.select()

            if len( fds[ 0 ] ) < 1 or len( fds[ 1 ] ) < 1:
                return True

            (command, tid, params) = self.socket.receiveCommand()
        except Exception, e:
            close(str(self) + ' error, closing (select/receive)\n' + str(e))
            return False
        
        if command == '':
            # empty command means some socket borkedness
            return False
        elif command == 'MSG':
            try:
                message = self.socket.receivePayload( int(params.split()[1]) )
            except Exception, e:
                close(str(self) + ' error, closing (payload)\n' + str(e))
                return False
                
            nick = params.split()[ 0 ]
            common.debug( message, 'switchboard' )
            try:
                header, body = self.splitMsg( message )
            except IndexError:
                common.debug("malformed message")

            Type = ''
            if "Content-Type" in header:
                Type = header["Content-Type"]
            
            if Type.startswith('text/plain'):
                # MSG god@gmail.com God%20http://www.400monkeys.com/God/ 111\r\n
                # MIME-Version: 1.0
                # Content-Type: text/plain; charset=UTF-8
                # X-MMS-IM-Format: FN=Sans; EF=; CO=000000; PF=0
                # 
                # a
                
                if not self.firstMessage:
                    self.firstMessage = True
                    self.msn.emit( 'new-conversation', self.firstUser, self )
                
                format = ''
                if 'X-MMS-IM-Format' in header:
                    format = header['X-MMS-IM-Format']
                
                try:
                    charset = Type.split('text/plain; charset=')[1]
                except IndexError:
                    charset = ''
                
                self.emit( 'message', tid, nick, body, format, charset )
                self.msn.emit( 'message-received', tid )

                # WLM doesn't reply invites before the actual MSG
                if tid in self.pendingCEs:
                    for mail, msnobj, name in self.pendingCEs[tid]:
                        self.getCustomEmoticon(mail, msnobj, name)
                    del self.pendingCEs[tid]
                
            elif Type == 'text/x-msnmsgr-datacast':
                # datacasts: nudges and actions
                if body.find('ID: 1') != -1:
                    # nudges
                    if not self.firstMessage:
                        self.firstMessage = True
                        self.msn.emit( 'new-conversation', self.firstUser, self )
                    
                    self.msn.emit( 'nudge-received', tid )
                    self.emit( 'nudge', tid )

                elif body.find('ID: 4') != -1:
                    # actions
                    if not self.firstMessage:
                        self.firstMessage = True
                        self.msn.emit( 'new-conversation', self.firstUser, self )
                    data = body.split("Data: ")[1] 
                    self.emit( 'action-message', tid, data )
                    self.msn.emit( 'message-received', tid )
            
            elif Type == 'text/x-msmsgscontrol':
                self.emit( 'typing', tid )

            elif Type == 'text/x-mms-emoticon' or \
                 Type == 'text/x-mms-animemoticon':
                if not self.firstMessage:
                    self.firstMessage = True
                    self.msn.emit( 'new-conversation', self.firstUser, self )
                self.parseCustomEmoticon( message )

            elif Type == 'application/x-msnmsgrp2p': 
                # aMSN 0.97b dont send the p2p-dest :S
                #and header['P2P-Dest'].lower() == self.user:
                if body.startswith("@\x00\x00\x00"):
                    try:
                        self.parseInkMessage( message, tid )
                    except Exception, e:
                        common.debug(str(e), 'switchboard')
                else:
                    p2p = self.msn.p2p[tid]
                    bin = msn_p2p.get_bin_header(body)
                    if msn_p2p.is_invite(message) or \
                        bin.identifier in p2p.pending_invites:
                        p2p.emit('invite-message-received', body)
                    else:
                        params = msn_p2p.get_parameters(body)
                        p2p.emit('msnp2p-message-received', \
                            bin, params, body)

            elif Type == 'image/gif':
                try:
                    self.parseInkMessage( message, tid )
                except Exception, e:
                    common.debug(str(e), 'switchboard')
            elif 'Message-ID' in header:
                try:
                    if header['Message-ID'] in self.inkSessions.keys():
                        self.parseInkMessage( message, tid )
                except Exception, e:
                    common.debug(str(e), 'switchboard')
            else:
                common.debug("Unhandled content type: " + str(header), 'switchboard')
                
        elif command == 'USR':
            if params.split( ' ' )[ 0 ] == 'OK':
                self.setStatus( 'connected' )
            else:
                close('can\'t connect to switchboard: USR ' + str(params) )

        elif command == 'IRO':
            #IRO 1 1 1 god@gmail.com God%20http://www.400monkeys.com/God/ 1615642660\r\n
            ( currentNumber, totalNumber, mail, nick, clientID ) = params.split( ' ' )
            self.addMember( mail.lower(), nick )
            
            self.emit( 'user-join', mail.lower() )
                
            self.setStatus( 'established' ) 
                
        elif command == 'JOI':
            # JOI god@gmail.com God%20http://www.400monkeys.com/God/ 1615642660\r\n
            ( nick, clientID ) = params.split()[:2] # likely to break..
            self.addMember( tid, nick )
            
            self.emit( 'user-join', tid )
                
            self.setStatus( 'established' ) 
                
        elif command == 'ANS':
            if params == 'OK':
                self.setStatus( 'connected' )
            else:
                close('can\'t connect to switchboard: ANS ' + str(params) )
        elif command == 'BYE':
            wasGroupChat = self.isGroupChat()
            self.leave( tid )
            
            # TODO, check when a bye means an OUT
            # from the other side.
            if params == '1' or wasGroupChat:
                self.emit( 'user-leave', tid )

        try:
            self.error = int(command)
            if len(self.members) == 0:
                close('server error %d, closing switchboard' % self.error, \
                    'error')
                return False
        except ValueError:
            pass

        return True
    
    def parseInkMessage( self, message, tid ):
        # TODO: integrate this with msn_p2p
        header, body = self.splitMsg( message )
        if body[0] == "@":
            # p2p ink on private conversation
            common.debug("p2p ink message", 'switchboard')
            p2p = True
            
            # get info
            binary = msn_p2p.BinHeader(body[:48])
            identifier = binary.identifier
            size = binary.total_data_size
            offset = binary.data_offset
            length = binary.message_length
            
            # get content
            content = body[48:-4].replace("\x00","") # TODO: find a better way
            
            if offset == 0:  # first message
                cheader, cbody = self.splitMsg( content )
                content = cbody[7:]
        else:
            # group chat, but it can be received on any situation
            # this is more efficient than the other method because there
            # is no utf-16 encoding (and that means double size)
            common.debug("group chat ink", 'switchboard')
            p2p = False
            
            if "Content-Type" in header :
                if body[0:7] != "base64:":
                    common.debug("base64 sig not found", 'switchboard')
                    return
                content = body[7:]
            else: 
                content = body
            length = 1
            offset = 0
            if "Message-ID" in header:
                # multipart
                identifier = header["Message-ID"]
                if "Content-Type" in header and "Chunks" in header:
                    size = int(header["Chunks"])
                else:
                    offset = int(header["Chunk"])
            else:
                # only one message, no sessions needed
                identifier = ''
                size = 1
                
        # save the session data
        if offset==0:
            inkData = content
            if identifier:
                self.inkSessions[identifier] = { \
                    'data': inkData, \
                    'total': size, \
                    'actual': length \
                }
                current = self.inkSessions[identifier]
                common.debug("ink: %s / %s" % \
                    (current['actual'], current['total']), 'switchboard')
        else:
            current = self.inkSessions[identifier]
            current['actual'] += length
            current['data'] += content
            common.debug("ink: %s / %s" % \
                (current['actual'], current['total']), 'switchboard')
            
        # check if transfer has finished
        if not identifier or (current['actual'] == current['total']):
            common.debug("saving...", 'switchboard')
            if identifier:
                inkData = current['data']
            inkData = base64.decodestring(inkData)
            
            filename = "ink%s_%s.gif" % (urllib.quote(tid), identifier)
            completeFileName = self.msn.cacheDir + os.sep + filename
            
            rc = open(completeFileName, "w")
            rc.write(inkData)
            rc.close()
            
            self.emit('ink-message', tid, completeFileName)
            self.msn.emit( 'message-received', tid )
            if p2p:
                ack = msn_p2p.BinHeader()
                ack.session_id = 64
                ack.identifier = random.randint(0,50000)
                ack.flag = msn_p2p.Base.ACK_FLAG
                ack.total_data_size = binary.total_data_size
                ack.acknowledged_data_size = binary.total_data_size
                ack.acknowledged_identifier = binary.identifier
                ack.acknowledged_unique_id = binary.acknowledged_identifier

                reply = ack.data + msn_p2p.Base.FOOTER
                self.sendMessage(reply, '', 'application/x-msnmsgrp2p', \
                    'D', 'P2P-Dest: ' + tid )
                common.debug("sent acknowledge", 'switchboard')
                
            if identifier: del self.inkSessions[identifier]

    def sendInk( self, buffer ):
        '''sends a gif file (in the file object "buffer") as an ink file'''
        self.emit('ink-sent')

        # p2p chunk size
        CHUNK_SIZE = 1202
        # generate an uuid
        messageId = '{' + str(uuid.uuid4()) + '}'
        
        # split the file in 1202 byte chunks
        chunks = []
        prepend = 'base64:'
        data = base64.encodestring(buffer.read())
        while len(data) > 0:
            size = CHUNK_SIZE - len(prepend)
            chunk, data = data[:size], data[size:]
            chunks.append(prepend + chunk)
            prepend = ''
        
        chunkNumber = 0
        for chunk in chunks:
            # build the message
            header = ''
            if chunkNumber == 0:
                contentType = 'image/gif'
                if len(chunks) > 1:
                    header = 'Message-ID: ' + messageId + '\r\n'
                    header += 'Chunks: ' + str(len(chunks)) + '\r\n'
            else:
                contentType = ''
                header = 'Message-ID: ' + messageId + '\r\n'
                header += 'Chunk: ' + str(chunkNumber) + '\r\n'

            # send the message
            self.sendMessage(chunk, '', contentType, 'A', header)
            chunkNumber += 1
        # done!

    def parseCustomEmoticon( self, message ):
        body = message.split( '\r\n\r\n' )[1]
        l = body.split('\t')
        d = {}
      
        while len(l) > 0:
            if len(l) < 2:
                break
            shortcut = l.pop(0)
            msnobjString = l.pop(0)
            msnobj = Msnobj.createFromString(msnobjString, False)
           
            if msnobj != None:
                self.emit('custom-emoticon-received', shortcut, msnobj)
                filename = shortcut + '_' + msnobj.sha1d + '.tmp'
                filename = urllib.quote(filename).replace('/', '_')
                completeFileName = self.msn.cacheDir + os.sep + filename
                
                if not os.path.isfile(completeFileName):
                    # WLM doesn't reply invites before the actual MSG
                    mail = msnobj.getCreator()
                    if mail not in self.pendingCEs:
                        self.pendingCEs[mail] = []
                    self.pendingCEs[mail].append((mail, msnobj, \
                        completeFileName))
                else:
                    self.msn.emit('custom-emoticon-transfered', None, \
                        msnobj, completeFileName)

    def sendMessage( self, msg='', format='', \
                     contentType='text/plain; charset=UTF-8', \
                     acknowledgeType='A', extraHeader='' ):
        
        header = "MIME-Version: 1.0\r\n"
        
        if contentType != "":
            header += "Content-Type: " + contentType + "\r\n"

        if extraHeader != "":
            header += extraHeader + '\r\n'
            
        if contentType == 'text/plain; charset=UTF-8':
            msg = msg[:1100] #TODO: use something like MAX_MESSAGE_LENGTH
            self.emit('message-sent', msg, format, 
                contentType.split('charset=')[1])
        
        if format != "":
            header += format + "\r\n\r\n"
        else:
            header += "\r\n"
        
        if self.status != 'established':
            self.messageQueue.append( {
                'msg': msg, \
                'header': header, \
                'acknowledgeType': acknowledgeType, \
            } )

        elif self.status == 'established':
            try:
                self.socket.sendPayloadCommand( 'MSG', acknowledgeType, \
                    header + msg )
            except Exception, e:
                #raise e
                print 'socket error on switchboard, closing switchboard'
                common.debug( str( e ), 'switchboard' )
                self.setStatus( 'closed' )

    def flushMessageQueue( self ):
        '''send all the unsended messages'''
        
        if self.status != 'established' or len(self.messageQueue) == 0:
            return
        
        for i in self.messageQueue:
            try:
                self.socket.sendPayloadCommand( 'MSG', \
                    i['acknowledgeType'], i['header'] + i['msg'] )
            except Exception, e:
                common.debug( 'socket error on switchboard, ' + \
                    'closing switchboard', 'switchboard' )
                common.debug( str( e ), 'switchboard' )
                self.setStatus( 'closed' )
        # flushed
        self.messageQueue = []
        common.debug( 'message queue flushed', 'switchboard' )

    def sendNudge( self ):
        '''a easy method to send a nudge'''

        self.sendMessage( 'ID: 1\r\n\r\n', '', 'text/x-msnmsgr-datacast' )
        self.emit('nudge-sent')

    def sendAction( self, data ):
        '''a easy method to send an action message - this works with MSNC6+'''

        self.sendMessage( 'ID: 4\r\nData: %s\r\n' % data, '', 'text/x-msnmsgr-datacast' )
        self.emit('action-sent', data)

    def sendIsTyping( self ):
        '''a easy method to send the is typing message'''

        self.sendMessage( '\r\n', '', 'text/x-msmsgscontrol\r\nTypingUser: ' + self.user )

    def invite( self, mail ):
        '''invite somebody to the switchboard conversation'''

        mail = mail.lower()
        if self.firstUser == '':
            self.firstUser = mail

        if self.status in ('connected', 'established') and \
           not mail in self.members.keys():
            try:
                self.socket.sendCommand( 'CAL', mail )
            except Exception, e:
                #raise e
                common.debug( 'socket error on switchboard, ' +
                    'closing switchboard', 'switchboard' )
                common.debug( str(e), 'switchboard' )
                self.setStatus( 'closed' )
                return
            
            if mail in self.invitationQueue:
                self.invitationQueue.pop( self.invitationQueue.index( mail ) )
        elif not mail in self.invitationQueue:
            self.invitationQueue.append( mail.lower() )

    def leave( self, mail ):
        '''remove the mail from the members list'''
        
        if mail.lower() in self.members.keys():
            if len( self.members ) == 1:
                self.setStatus('closed')
            else:
                del self.members[ mail.lower() ]
    
    def setConnectionString( self, connectionString ):
        '''we received the conecction string (the first response of a XFR
        or the RNG) and here we process it'''
        
        self.connectionString = connectionString        
        self.parse()
        self.connectSocket()
        
    def connectSocket( self ):
        '''connect the socket'''

        if self.proxy != None:
            self.socket = Socket.PSocket( self.host, self.port, \
                self.proxy, 'SB' )
            self.logoutId = self.msn.connect('logout', self.socket.quit)
            # we don't need to connect some process timeout here
            # because it's done in core.httpProcess
        else:
            self.socket = Socket.Socket( self.host, self.port )
            self.socket.settimeout(5)
            try:
                self.socket.connect()
            except socket.error, e:
                common.debug( "can't connect to switchboard, socket error", \
                    'switchboard' )
                common.debug( str( e ), 'switchboard' )
                self.setStatus( 'closed' )
                return
            else:
                self.socket.settimeout(None)
                gobject.idle_add(self.addWatches)
        
        if self.command == 'XFR':
            try:
                self.socket.sendCommand( "USR" , self.user + " " + \
                    self.authenticationString )
            except Exception, e:
                #raise e
                common.debug( 'socket error on switchboard, ' + \
                    'closing switchboard', 'switchboard' )
                common.debug( str( e ), 'switchboard' )
                self.setStatus( 'closed' )
            
        else: #RNG
            # ANS 1 alice@passport.com 1056411141.26158 17342299\r\n
            try:
                self.socket.sendCommand( "ANS" , self.user + " " + \
                    self.authenticationString + " " + self.sessionID )
            except Exception, e:
                #raise e
                common.debug( 'socket error on switchboard, ' + \
                    'closing switchboard', 'switchboard' )
                common.debug( str( e ), 'switchboard' )
                self.setStatus( 'closed' )
        
    def addWatches( self ):
        '''Connects the I/O add watch signals'''
        if os.name == 'nt':
            self.signals.append(gobject.timeout_add(200, self.process))
        else:
            self.signals.append(gobject.io_add_watch(self.socket, \
                gobject.IO_IN, self.process))
            self.signals.append(gobject.io_add_watch(self.socket, \
                gobject.IO_ERR | gobject.IO_HUP, self.socketHangup))
        return False

    def addMember( self, mail, nick ):
        '''add a member to the members dict'''
        
        if self.firstUser == '':
            self.firstUser = mail.lower()
        self.members[ mail.lower() ] = nick.replace( '%20', ' ' )
        self.getDisplayPicture( mail.lower() )

    def setStatus( self, status ):
        '''set the status, the status can be 'pending', 'stablished' or 'closed'
        PLEASE change the status here and not directly because we may want
        to do thing when a status is changed
        XXX-DX: i think we need a property here, we can't ask "PLEASE" that
        way...'''
        
        if status in self.validStatus and self.status != status:
            if self.invalidTransitions.has_key( self.status ) and \
               status in self.invalidTransitions[ self.status ]:
                return

            self.status = status

            if status in ('closed', 'error'):
                self.msn.disconnect( self.userOfflineId )
                self.msn.disconnect( self.userOnlineId )
                if self.proxy != None:
                    self.msn.disconnect( self.logoutId )
                    self.socket.quit()

                for i in self.signals:
                    gobject.source_remove(i)
                
                self.signals = []
            if status == 'error':
                try:
                    self.socket.sendText( 'OUT\r\n' )
                except Exception, e:
                    pass
                self.socket = None
            if status == 'connected':
                for i in self.invitationQueue:
                    self.invite( i )
                    
                self.invitationQueue = []
            elif self.status == 'established':
                self.flushMessageQueue()
            
            self.emit( 'status-change' )
        
    def getId( self ): #FIXME: getter
        '''return the id of the switchboard, this is a unique identification
        of the switchboards. The value is the value of the Trid of the
        command that created the switchboard or the sessionid if the other
        user started the conversation, the value doesnt matter, what
        matters is that its unique.'''
        
        return self.id
    
    def getOnlineUsers( self ):
        '''This method returns a list ol mails of the contacts who are
        not offline'''
        
        return self.msn.contactManager.getOnlineUsers()
    
    def getMembers( self ):
        '''return a list of the members in the switchboard'''
        
        return self.members.keys()

    def isGroupChat( self ):
        return (len( self.members ) > 1)
        
    def getInvitedMembers( self ):
        '''return a list of the members invited but not joined'''
        
        return self.invitationQueue
        
    def getUserDisplayName( self, mail ):
        '''return the user display name ot just the mail if it cant be found'''
        
        alias = self.msn.contactManager.getContactAlias( mail.lower() )
        
        if alias:
            return alias
        else:
            return self.msn.contactManager.getContactNick( mail.lower() )

    def userOffline( self, msnp, email ):
        '''Method called when a user get offline'''
        email = email.lower()
        if email in self.members.keys():
            self.emit( 'user-offline', email )
            self.leave( email )

    def userOnline( self, msnp, email, oldStatus ):
        '''Method called when a user get online'''
        email = email.lower()
        if email in self.members.keys() and oldStatus == 'FLN':
            self.emit( 'user-online', email )
        
    def getNick(self, mail, use_alias_if_available=True):
        '''return the nick of the user mail'''
        
        if use_alias_if_available:
            return self.msn.contactManager.getContactNameToDisplay(mail.lower())
        else:
            return self.msn.contactManager.getContactNick(mail.lower())
    
    def getDisplayPicture( self, email ):
        '''start a P2P session to get the display picture'''
        email = email.lower()
        contact = self.msn.contactManager.getContact( email )
        
        if contact == None:
            common.debug( 'contact (' + email + ') not found in ' + \
                'getDisplayPicture', 'switchboard' )
            return
        
        msnobj = contact.msnobj
        
        if msnobj == None:
            common.debug( email + ' has no msnobj in getDisplayPicture', \
                'switchboard' )
            return

        #print "Switchboard.getDisplayPicture(email=%s)" % email
        
        filename = os.path.join(self.msn.cacheDir, contact.displayPicturePath)
        if not os.path.exists(filename):
            #print "Requesting avatar for ", email
            msn_p2p.Receiver(self.msn.p2p[email], self.user, msnobj)
        else:
            #print "Avatar cached at %s - updating path" % filename
            self.msn.emit("display-picture-changed", self, msnobj, email)
            
    def getCustomEmoticon( self, email, msnobj, filename ):
        email = email.lower()
        msnobj.filename = filename
        msn_p2p.Receiver(self.msn.p2p[email], self.user, msnobj)
        
    def sendCustomEmoticons( self, message ):
        msnObjs = []
        msnObj=''
        i=0
        msnOM = self.msn.getMsnObjectsManager()
        for CE in msnOM.getIds():
            if i == 4:
                i=0
                msnObjs.append( msnObj )
                msnObj = ''
            if message.find(CE) != -1:
                msnObj += CE+'\t'+str( msnOM.getById(CE) )+'\t'
                i +=1
        if msnObj != '': msnObjs.append( msnObj )
        for msnObj in msnObjs:
            self.sendMessage( msnObj, contentType='text/x-mms-animemoticon' )

gobject.type_register(Switchboard)
