# -*- coding: utf-8 -*-

#   This file is a plugin for emesene.
#
#    Dbus Emesene Plugin is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Dbus Emesene Plugin is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import Plugin
try:
    import dbus
    dbusError = ''
    if getattr(dbus, 'version', (0,0,0)) >= (0,41,0):
        import dbus.glib
except Exception, e:
    dbusError = e

from emesenecommon import *

    
class Dbus:

    def __init__( self , theme ):
    
        self.theme = theme

class MainClass( Plugin.Plugin ):
    '''Main plugin class'''
    
    def __init__( self, controller, msn ):
        '''Contructor'''
        
        Plugin.Plugin.__init__( self, controller, msn )
        self.theme = controller.theme
        
        self.description = _( 'With this plugin you can interact via Dbus with emesene' )
        self.authors = { 'Roger Duran' : 'RogerDuran at gmail dot com' }
        self.website = 'http://www.rogerpc.com.ar'
        self.displayName = _( 'Dbus' )
        self.name = 'Dbus'
        
        #callback ids
        self.userConnectId = None
        self.userOfflineId = None
        self.selfNickChangedId = None
        self.personalMessageChangedId = None
        
    def start( self ):
        '''start the plugin'''
        
        try:
            self.startDbus()
        except:
            return
        
        self.userConnectId = self.connect( 'user-online', self.userConnect )
        self.userOfflineId = self.connect( 'user-offline', self.userOffline )
        self.selfNickChangedId = self.connect( 'self-nick-changed', self.selfNickChanged )
        self.personalMessageChangedId = self.connect( 'personal-message-changed', self.personalMessageChanged )

        self.enabled = True
        
    def stop( self ):    
        '''stop the plugin'''
        self.disconnect( self.userConnectId )
        self.disconnect( self.userOfflineId )
        self.disconnect( self.selfNickChangedId )
        self.disconnect( self.personalMessageChangedId )

        self.enabled = False
        
    def action( self ):
        pass
    
    def check( self ):
        '''
        check if everything is OK to start the plugin
        return a tuple whith a boolean and a message
        if OK -> ( True , 'some message' )
        else -> ( False , 'error message' )
        '''
        if dbusError != '':
            return ( False, 'Can\'t Load dbus', dbusError )
            
        return ( True, 'Ok' )


    def startDbus(self):
        '''Start dbus session'''
        self.session_bus = dbus.SessionBus()
        self.bus_name = dbus.service.BusName('org.gnome.Emesene', bus=self.session_bus)
        self.dbusObject = EmeseneObject(self.bus_name, '/org/gnome/EmeseneObject', self)

    def userConnect(self, msnp, user, oldStatus):
        if(oldStatus=='FLN'):
            self.dbusObject.userConnect(user)


    def userOffline(self, msnp, user):
        self.dbusObject.userOffline(user)

    def selfNickChanged(self, msnp, old, nick):
        self.dbusObject.selfChangeNick(nick)

    def personalMessageChanged(self, msnp, email, pm):
        self.dbusObject.personalMessageChanged(email, pm)
try:
    class EmeseneObject(dbus.service.Object):
        def __init__(self, bus_name, object_path='/org/gnome/EmeseneObject', main=''):
            dbus.service.Object.__init__(self, bus_name, object_path)
            self.main = main
            self.msn = main.msn
            self.controller = main.controller
        #Methods
        @dbus.service.method('org.gnome.EmeseneIFace')
        def changePersonal(self, text):
            self.msn.changePersonalMessage( text )
            return 'Personal Changed'

        @dbus.service.method('org.gnome.EmeseneIFace')
        def changeCurrenMedia(self, text):
            self.msn.changeCurrentMedia( text )
            return 'Current Media Changed'

        @dbus.service.method('org.gnome.EmeseneIFace')
        def changeStatus(self, status):
            self.msn.changeStatus(status)
            return 'Status Changed'

        @dbus.service.method('org.gnome.EmeseneIFace')
        def changeNick(self, nick):
            self.msn.changeNick(nick)
            return 'Nick changed'

        @dbus.service.method('org.gnome.EmeseneIFace')
        def startNewConversation(self, email=''):
            try:
                self.controller.newConversation( None, email )
                return 'opened ' + str(email)
            except Exception, e:
                return 'error trying to start a conversation: ' + str( e )

        @dbus.service.method('org.gnome.EmeseneIFace')
        def getOnlineUsers(self):
            users = self.main.controller.msn.contactManager.getOnlineUsers()
            contacts = []
            for user in users:
                name = self.controller.msn.contactManager.getContactDisplayName(user[0])
                contact = {'email': user[0], 'name': name, 'status': user[1]}
                contacts.append(contact)
            return contacts

        @dbus.service.method('org.gnome.EmeseneIFace')
        def getThemePath(self):
            theme = self.controller.config.user['theme']
            themePath = os.path.expanduser("~") + "/.config/emesene1.0/themes/" + theme
            return themePath

        @dbus.service.method('org.gnome.EmeseneIFace')
        def getContactNick(self, email):
            return self.controller.msn.contactManager.getContactDisplayName(email)
            
        @dbus.service.method('org.gnome.EmeseneIFace')
        def getContactPersonalMessage(self, email):
            return self.controller.msn.contactManager.getContactPersonalMessage(email)
            
        #Signals
        @dbus.service.signal('org.gnome.EmeseneIFace')
        def userConnect(self, user):
            pass

        @dbus.service.signal('org.gnome.EmeseneIFace')
        def userOffline(self, email):
            pass

        @dbus.service.signal('org.gnome.EmeseneIFace')
        def selfChangeNick(self, nick):
            pass

        @dbus.service.signal('org.gnome.EmeseneIFace')
        def personalMessageChanged(self, email, pm):
            pass
except Exception, e:
    dbusError = e
