# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import gtk
import Plugin
import desktop
import emesenelib.common
from Parser import UserListDataType

ERROR = ''
try:
    import pynotify
    if not pynotify.init("emesene"):
        raise
except:
    ERROR = _( 'there was a problem initializing the pynotify module' )
    
class MainClass( Plugin.Plugin ):
    '''Main plugin class'''
    
    def __init__( self, controller, msn ):
        '''Contructor'''
        
        Plugin.Plugin.__init__( self, controller, msn )
        
        self.description = _( 'Notify about diferent events using pynotify' )
        self.authors = {'Mariano Guerra': 'luismarianoguerra at gmail dot com'}
        self.website = 'http://emesene-msn.blogspot.com'
        self.displayName = _( 'LibNotify' )
        self.name = 'LibNotify'
        
        self.tray = self.controller.trayIcon.getNotifyObject()
        self.pixbuf = self.controller.theme.getImage( 'userPanel' )
        
        self.controller = controller
        self.parser = controller.unifiedParser
        
        self.notifications = []
        
        # callbacks ids
        self.onlineId = None
        self.newMailId = None
        
    def start( self ):
        '''start the plugin'''
        
        self.onlineId = self.connect( 'user-online', self.online )
        self.newMailId = self.connect( 'new-mail-notification', self.newMail )
        self.enabled = True
        
    def stop( self ):    
        '''
        stop the plugin
        '''
        
        # closes remaining notifications
        for i in self.notifications:
            i.close()

        self.notifications = []
            
        self.disconnect( self.onlineId )
        self.disconnect( self.newMailId )
        self.enabled = False
        
    def check( self ):
        '''check if everything is OK to start the plugin
        return a tuple whith a boolean and a message
        if OK -> ( True , 'some message' )
        else -> ( False , 'error message' )'''
        global ERROR
        
        if ERROR != '':
            return ( False, ERROR )
        
        return ( True, 'Ok' )
    
    def online( self, msnp, email, oldStatus ):
        '''called when someone get online'''

        if oldStatus != 'FLN':
            return

        contact = self.msn.contactManager.getContact( email )
        text = ' - '.join(self.controller.mainWindow.userList.getContactLabel(\
            contact) )

        if self.controller.theme.hasUserDisplayPicture( contact ):
            self.pixbuf = self.controller.theme.getUserDisplayPicture( \
                contact, 48, 48 )
        
        notification = pynotify.Notification("emesene", \
            text + " (" + email + ')\n' + _("is now online"), \
            attach=self.tray )

        notification.set_icon_from_pixbuf( self.pixbuf )
#        notification.add_action( 'default', 'Open conversation', \
#                                 self.on_notify_action )

        if self.controller.trayIcon and \
           isinstance(self.controller.trayIcon.tray, gtk.StatusIcon):
            tray = self.controller.trayIcon.tray
            try:
                notification.attach_to_status_icon(tray)
                #notification.set_hint("x", tray.get_geometry()[1].x)
                #notification.set_hint("y", tray.get_geometry()[1].y)
            except:
                print "cant set geometry hints on libnotify plugin"
            
        notification.show()
        notification.connect( 'closed', self.on_notify_close )
        self.notifications.append( notification )

        self.pixbuf = self.controller.theme.getImage( 'userPanel' )

    def newMail(self, msnp, From, FromAddr, Subject, MessageURL, PostURL, id):
        text = _('From: ')+From +' &lt;'+FromAddr+'&gt;\n'
        text += _('Subj: ') + emesenelib.common.escape( Subject )
        text += '\n\n<span foreground="#AAAAAA">emesene</span>'
        
        def openMail( notification, action ):
            desktop.open( \
                self.controller.hotmail.getLoginPage(MessageURL, PostURL, id))
        
        notification = pynotify.Notification(_( 'New email' ), \
                              text, \
                              attach=self.tray )
        notification.set_icon_from_pixbuf( self.pixbuf )
        notification.add_action( 'default', 'Open Mail', \
                                 openMail )
        if self.controller.trayIcon and \
           isinstance(self.controller.trayIcon.tray, gtk.StatusIcon):
            tray = self.controller.trayIcon.tray
            try:
                notification.attach_to_status_icon(tray)
            except:
                print "cant set geometry hints on libnotify plugin"

        notification.show()
        notification.connect( 'closed', self.on_notify_close )
        self.notifications.append( notification )
    
    def on_notify_close( self, n ):
        if n in self.notifications:
            self.notifications.remove( n )

#    def on_notify_action( self, notification, action ):
#        print ref
#        print str[:]
#        self.controller.newConversation( '' )

