# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import gtk
import time
import gobject
import pango

import Plugin
from emesenelib.common import escape
from emesenecommon import PATH
from Theme import resizePixbuf
from ImageFileChooser import ImageFileChooserDialog

growFactor = 20 # the number of pixels to grow every iteration

class PixmapDialog( gtk.Dialog ):
    '''a dialog to set Notification Pixmap'''
    def __init__( self, parent, config, filename, font , color ):
        gtk.Dialog.__init__( self , _('Notification Config'), parent , \
                gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT , \
                (gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT, \
                gtk.STOCK_ADD, gtk.RESPONSE_ACCEPT) )
        
        self.config = config
        self.filename = filename
        self.fontdesc = font
        self.FColor = color

        self.set_default_response( gtk.RESPONSE_ACCEPT )
        
        self.hbox = gtk.HBox()
        self.lbox = gtk.HBox()

        self.image = gtk.Image()
        
        self.sample = gtk.Label()
        self.sample.set_label( '<span foreground="%s">%s</span>' % \
            ( self.FColor, _('Sample Text') ))
        self.sample.set_use_markup( True )

        try:
            self.sample.modify_font( pango.FontDescription( self.fontdesc ) )
        except:
            print 'Font Error'
        
        self.fonttype = gtk.ToolButton()
        self.fonttype.set_stock_id(gtk.STOCK_SELECT_FONT)
        
        self.fontColor = gtk.ToolButton()
        self.fontColor.set_stock_id(gtk.STOCK_SELECT_COLOR)

        try:
            pixbuf =  gtk.gdk.pixbuf_new_from_file( self.filename )
            pixbuf = resizePixbuf( pixbuf, 24,24 )
            self.image.set_from_pixbuf( pixbuf )
        except:
            self.filename = None

        self.button = gtk.Button( _('Image') )

        self.hbox.pack_start( self.image )
        self.hbox.pack_start( self.button )
        
        self.lbox.pack_start( self.fonttype )
        self.lbox.pack_start( self.fontColor )

        self.button.connect( 'clicked', self.clickPixmap )
        self.fonttype.connect('clicked', self.clickFont )
        self.fontColor.connect('clicked', self.clickColor )

        self.vbox.pack_start( self.hbox )
        self.vbox.pack_start( self.sample )
        self.vbox.pack_start( self.lbox )

        self.vbox.show_all()

    def clickPixmap( self, arg ):
        self.filename = ImageFileChooserDialog( self.config ).run()[1]
        try:
            pixbuf = gtk.gdk.pixbuf_new_from_file( self.filename )
            pixbuf = resizePixbuf( pixbuf, 24,24 )
            self.image.set_from_pixbuf( pixbuf )
            self.image.show()
        except:
            self.filename = None
            self.image.hide() 
   
    def clickFont( self, arg ):
        fontDialog = gtk.FontSelectionDialog( _( 'Choose a font' ) )
        if self.fontdesc != None:
            fontDialog.set_font_name( self.fontdesc )
        response = fontDialog.run()
        if response == gtk.RESPONSE_OK:
            pangoDesc = pango.FontDescription( fontDialog.get_font_name() )
            self.sample.modify_font(pangoDesc)
            self.fontdesc = pangoDesc.to_string()
        fontDialog.destroy()
    
    def clickColor( self, arg):
        colorDialog = gtk.ColorSelectionDialog( _( 'Choose a color' ) )
        colorDialog.colorsel.set_has_palette( True )
        response = colorDialog.run()
        if response == gtk.RESPONSE_OK:
            color = colorDialog.colorsel.get_current_color()
            red = color.red >> 8
            green = color.green >> 8
            blue = color.blue >> 8
            self.FColor = '#%02X%02X%02X'  % ( red , green , blue )
            self.sample.set_label( '<span foreground="%s">%s</span>' % ( self.FColor, _('Sample Text') ))
            self.sample.set_use_markup( True )
        colorDialog.destroy()

    def run( self ):
         option = gtk.Dialog.run( self )
         self.destroy()   
         return ( self.filename, self.fontdesc, self.FColor )

class NotificationManager:
    '''
    This class manages the creation display and destruction of the notifications.
    '''

    def __init__( self , defaultHeight = 128 , defaultWidth = 200 ):
        '''
        Contructor
        '''
        
        self.defaultHeight = defaultHeight
        self.defaultWidth = defaultWidth
        
        #[ [ Notification , timestamp ] , [ Notification , timestamp ] ]
        self.list = []
        
    def newNotification( self , string , email, pixmap = None, buttonimage = None, callback = None, userPixbuf = None, font = None, color = None ):
        '''
        create a new notification, pixmap is the background image (as a pixbuf),
        imagebutton is a gtk.Image for the close button.
        callback is the method that will be called when the message in the Notification
        is clicked
        '''
        if pixmap != None:
            width, height = pixmap.get_size()
        else:
            width = self.defaultWidth
            height = self.defaultHeight

        g = Notification( string , email, self.compact() , height, width, pixmap, buttonimage, callback, userPixbuf, font, color )
        g.show()
        
        self.list.append( [ g , int( time.time() ) ] )
        
    def check( self ):
        '''
        check which notifications should be closed,
        this method should be called 1 time per second (recommended)
        '''

        if self.list == []:
            return True
        else:
            timestamp = int( time.time() )
            count = 0
            for i in self.list:

                if i[ 1 ] + 5 <= timestamp:
                    i[ 0 ].hide()
                    del self.list[ count ]
                else:
                    self.list[ count ][ 0 ].grow()
                
                count += 1
                    
        return True
        
    def compact( self ):
        '''
        make the notifications be one on top of eachother and return the next position
        '''

        for i in range( len( self.list ) ):
            self.list[ i ][ 0 ].move( i )
            
        return len( self.list )
    
    def closeAll( self ):
        '''
        close all the notiications
        '''
        
        for i in range( len( self.list ) ):
            self.list[ i ][ 0 ].hide()
            
        self.list = []

class Notification( gtk.Window ):
    
    def __init__( self , string , email, position = 0 , height = 128 , width = 200 , pixmap = None, buttonImage = None, callback = None , userPixbuf = None, font = None, color = None ):
        
        gtk.Window.__init__( self )

        self.set_gravity(gtk.gdk.GRAVITY_SOUTH_EAST)

        self.set_property( 'can-focus', False )
        self.set_property( 'accept-focus', False )

        self.width = width
        self.maxHeight = height
        self.height = 1
        self.callback = callback

        self.positionNum = position
        
        self.resize( self.width, self.height )
        self.set_geometry_hints( None, min_width=-1, min_height=-1, max_width=self.width, max_height=self.maxHeight )
        self.set_accept_focus( False )
        
        self.set_decorated( False )
        self.set_keep_above( True )
        self.set_skip_taskbar_hint( True )
        self.set_skip_pager_hint ( True )
        
        if pixmap != None:
            self.set_app_paintable( True )
            self.resize(self.width, self.height)
            self.realize()
            self.window.set_back_pixmap( pixmap, False )
            
        messageLabel = gtk.Label( '<span foreground="' + color +'">' + escape(str(string)) + '</span>' )
        messageLabel.set_use_markup( True )
        messageLabel.set_justify( gtk.JUSTIFY_CENTER )
        #messageLabel.set_line_wrap( True )
        messageLabel.set_ellipsize( pango.ELLIPSIZE_END )
        try:
            messageLabel.modify_font( pango.FontDescription(font) )
        except e:
            print e

        if buttonImage == None:
            close = gtk.Label()
            close.set_label( "<span background=\"#cc0000\" foreground=" + color + "\"> X </span>" )
            close.set_use_markup( True )
        else:
            close = buttonImage

        closeEventBox = gtk.EventBox()
        closeEventBox.set_visible_window(False)
        closeEventBox.set_events(gtk.gdk.BUTTON_PRESS_MASK)
        closeEventBox.connect("button_press_event" , self.close)        
        closeEventBox.add(close)

        hbox = gtk.HBox()
        vbox = gtk.VBox()
        lbox = gtk.HBox()
        title = gtk.Label( "" )
        title.set_use_markup( True )
        
        avatarImage = gtk.Image()
        userPixbuf = resizePixbuf( userPixbuf, 48, 48)
        avatarImage.set_from_pixbuf( userPixbuf )

        lboxEventBox = gtk.EventBox()
        lboxEventBox.set_visible_window(False)
        lboxEventBox.set_events(gtk.gdk.BUTTON_PRESS_MASK)
        lboxEventBox.connect("button_press_event", self.onClick, email)
        lboxEventBox.add(lbox)

        hbox.pack_start( title , True , True )
        hbox.pack_end( closeEventBox, False, False )
        lbox.pack_start( avatarImage, False, False, 10 )
        lbox.pack_start( messageLabel, True , True, 5 )

        vbox.pack_start( hbox , False , False )
        vbox.pack_start( lboxEventBox , True , True )
        
        self.move( self.positionNum )
        self.add( vbox )
        vbox.show_all()
        
    def onClick( self, widget, event, email):
        if event.button == 3:
            self.close()
        else:
            self.callback(email)
        
    def resize( self , width , height ):
        '''
        change the size    
        '''

        gtk.Window.resize( self , width , height )    
        
        self.width = width
        self.height = height

        gtk.Window.move( self,gtk.gdk.screen_width() - self.width, gtk.gdk.screen_height() - self.height - (self.maxHeight * self.positionNum ) - 24 - ( self.positionNum * 2 ) )
        
    def show( self ):
        '''
        show it
        '''

        gtk.Window.show( self )
        
    def move( self , position ):
        '''
        change the position of the Notification
        '''

        gtk.Window.move( self,gtk.gdk.screen_width() - self.width, gtk.gdk.screen_height() - self.height - (self.maxHeight * position ) - 24 - ( position * 2 ) )
        #gtk.Window.move( self , gtk.gdk.screen_width() - self.width, gtk.gdk.screen_height() - (self.maxHeight * position ) - 24 - ( position * 2 ) )        
        self.positionNum = position
        
    def close( self , *args ):
        '''
        hide the Notification
        '''

        self.hide()

    def grow( self ):
        '''
        increase the size of the notification
        '''

        if self.height < self.maxHeight:
            if self.height + growFactor > self.maxHeight:
                self.height = self.maxHeight
                self.resize( self.width, self.height )
            else:
                self.height += growFactor
                self.resize( self.width, self.height  )

class MainClass( Plugin.Plugin ):
    '''
    The notification plugin
    '''
    
    def __init__( self, controller, msn ):
        '''
        Constructor
        '''
        
        Plugin.Plugin.__init__( self, controller, msn, 100 )
        self.theme = controller.theme
        
        self.config = controller.getConfig()

        self.description = _( 'Show a little window in the bottom-right corner of the screen when someone gets online etc.' )
        self.authors = { 'Mariano Guerra' : 'luismarianoguerra at gmail dot com' }
        self.website = 'http://emesene-msn.blogspot.com'
        self.displayName = _( 'Notification' )
        self.name = 'Notification'
       
        self.config.readPluginConfig(self.name)
        self.to = None
        
        self.onlineId  = None
        
    def online( self, msnp, email, oldStatus ):
        '''
        called when someone get online
        '''

        if oldStatus != 'FLN':
            return
        
        try:
            self.close = gtk.Image()
            self.close.set_from_pixbuf( self.theme.getImage( 'close' ) )
        except:
            self.close = None
        
        nick = self.msn.contactManager.getContactNick( email )
        contact = self.msn.contactManager.getContact( email )
        userPixbuf = self.theme.getUserDisplayPicture( contact )

        self.notificationManager.newNotification( unicode(nick)[ :20 ]  + "\n" +
                              _( "is online" ), email, self.pixmap, self.close, 
                              self.startConversation,userPixbuf,self.fontname, self.fontcolor )
        
    def startConversation( self, email ):
        
        try:
            self.controller.newConversation( self.controller.msn, email, None, True )
        except Exception, e:
            print 'error trying to start a conversation: ' + str( e )
        
    def start( self ):
        '''
        start the plugin
        '''
        
        self.notificationManager = NotificationManager( 128, 200 )
        self.onlineId  = self.connect( 'user-online', self.online )
        self.to = gobject.timeout_add( self.period, self.action )
        self.filename = self.config.getPluginValue(self.name, 'filename', PATH.DEFAULT_THEME_PATH + 'guif.png')
        self.fontname  = self.config.getPluginValue(self.name, 'fontname', 'Sans')
        self.fontcolor = self.config.getPluginValue(self.name, 'fontcolor', '#000000') 

        try:
            self.pixmap, mask = gtk.gdk.pixbuf_new_from_file(self.filename).render_pixmap_and_mask()
        except:
            self.pixmap = None
            
        try:
            self.close = gtk.Image()
            self.close.set_from_pixbuf( self.theme.getImage( 'close' ) )
        except:
            self.close = None
            
        self.enabled = True
        
    def stop( self ):    
        '''
        stop the plugin
        '''
        
        self.disconnect( self.onlineId )
        if self.to:
            gobject.source_remove( self.to )
        self.enabled = False
        self.notificationManager.closeAll()
    
    def action( self ):
        if self.enabled:
            self.notificationManager.check()
            return True
        else:
            return False
        
    def check( self ):
        '''
        check if everything is OK to start the plugin
        return a tuple whith a boolean and a message
        if OK -> ( True , 'some message' )
        else -> ( False , 'error message' )
        '''
        
        return ( True, 'Ok' )
    
    def configure( self ):
        self.filename,self.fontname,self.fontcolor = PixmapDialog( None, self.config, self.filename, self.fontname,self.fontcolor ).run()
        self.config.setPluginValue( self.name, 'filename', self.filename )
        self.config.setPluginValue( self.name, 'fontname', self.fontname )
        self.config.setPluginValue( self.name, 'fontcolor', self.fontcolor )
        try:
            self.pixmap, mask = gtk.gdk.pixbuf_new_from_file(self.filename).render_pixmap_and_mask()
        except:
            self.pixmap = None
        return True
