# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import os
import gtk
import stat
import urllib
import urllib2
import tempfile
import mimetools
import mimetypes
import threading

import Plugin

class MainClass(Plugin.Plugin):
    '''Main plugin class'''

    def __init__( self, controller, msn ):
        '''Constructor'''

        Plugin.Plugin.__init__( self, controller, msn )
        self.description = _( 'Take screenshots and upload to imageshack' )
        self.authors = { 'Dx' : 'dx@dxzone.com.ar' }
        self.website = 'http://www.dxzone.com.ar'
        self.displayName = 'Screenshack'
        self.name = 'Screenshack'
        
        self.config = controller.getConfig()
        self.config.readPluginConfig(self.name)

        self.controller = controller
        self.slash = controller.Slash
        self.msn = msn
        
        if os.name == 'posix':
            self.root = controller.mainWindow.get_root_window()
        else:
            self.root = None
        
        self.opener = urllib2.build_opener(MultipartPostHandler)

    def start( self ):
        '''start the plugin'''
        self.slash.register('screenshot', self.screenshot, \
            _('Takes screenshots') )
        self.slash.register('imageshack', self.imageshack, \
            _('Uploads images') )

        self.enabled = True

    def screenshot( self, slashAction ):
        params = slashAction.getParams()

        if params == 'save':
            ScreenThread( slashAction, self, save=True ).start()
        else:
            if self.config.getPluginValue(self.name, 'tip', '0') == '0':
                slashAction.outputText( \
                    _('Tip: Use "/screenshot save" to skip the upload '))
                self.config.setPluginValue(self.name, 'tip', '1')
            ScreenThread( slashAction, self ).start()

    def imageshack( self, slashAction ):
        path = slashAction.getParams()
        if type(path) == str and \
            os.path.exists(path) and \
            os.path.isfile(path):

            UploadThread( slashAction, self, path ).start()
        else:
            slashAction.outputText( _('File not found') )

    def upload( self, fileResource ):
        params = { 'fileupload' : fileResource, 'xml' : 'yes' }
        rc = self.opener.open("http://imageshack.us", params)
        try:
            text = rc.read()
            return text.split('<image_link>')[1].split('</image_link>')[0]
        except:
            return None

    def stop( self ):    
        '''stop the plugin'''
        self.slash.unregister('screenshot')
        self.slash.unregister('imageshack')
        self.enabled = False

    def check( self ):
        return ( True, 'Ok' )

class ScreenThread(threading.Thread):

    def __init__(self, action, plugin, save=False):
        threading.Thread.__init__( self )
        self.slashAction = action
        self.plugin = plugin
        self.save = save

    def run(self):
        if os.name == 'nt':
            print 'screenshot on windows never tested, check source!'

            # yes, i had to break this link.. 80 cols :P
            ## mail.python.org/pipermail/python-win32/
            ## .../2003-January/000709.html

            # import win32api, win32clipboard
            # win32api.keybd_event(win32con.VK_PRINT, 0)
            # win32clipboard.OpenClipboard(0)
            # data = win32clipboard.GetClipboardData(win32clipboard.CF_DIB)
            # win32clipboard.CloseClipboard()
        elif os.name == 'posix':
            temp = tempfile.mkstemp(suffix='.png')
            os.close(temp[0])
            
            # überfast screenshots! 1.9secs in my pentium II 350mhz
            # er.. yes, that's fast.. try running import
            root = self.plugin.root
            x, y = root.get_size()
            colormap = root.get_colormap()
            pixmap = gtk.gdk.pixmap_foreign_new(root.xid)
            pixbuf = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, 0, 8, x, y)
            pixbuf.get_from_drawable(pixmap, colormap, 0, 0, 0, 0, x, y)
            pixbuf.save(temp[1], 'png')
            
        if not self.save:
            url = self.plugin.upload( open(temp[1], 'rb') )
            os.remove(temp[1])
            if url == None:
                self.slashAction.outputText( _('Error uploading image') )
            else:
                self.slashAction.outputText( url, True )
        else:
            self.slashAction.outputText( _('Temporary file:') + ' ' + \
                temp[1] )

class UploadThread(threading.Thread):
    def __init__(self, action, plugin, path):
        threading.Thread.__init__( self )
        self.slashAction = action
        self.plugin = plugin
        self.path = path

    def run(self):
        rc = open(self.path, 'rb')
        url = self.plugin.upload(rc)
        self.slashAction.outputText(url, True)


#here follows some code from:
# http://odin.himinbi.org/MultipartPostHandler.py
#and:
# http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/146306
#with some style changes to fit to 80 cols

class Callable:
    def __init__(self, anycallable):
        self.__call__ = anycallable

# Controls how sequences are uncoded. If true, elements may be given
# multiple values by assigning a sequence.
DOSEQ = 1


class MultipartPostHandler(urllib2.BaseHandler):
    # needs to run first
    handler_order = urllib2.HTTPHandler.handler_order - 10

    def http_request(self, request):
        data = request.get_data()
        if data is None or type(data) == str:
            return request

        v_files = []
        v_vars = []
        try:
            for(key, value) in data.items():
                if type(value) == file:
                    v_files.append((key, value))
                else:
                    v_vars.append((key, value))
        except TypeError:
            systype, value, traceback = sys.exc_info()
            raise TypeError, "not a valid non-string sequence or " \
                "mapping object", traceback

        if len(v_files) == 0:
            data = urllib.urlencode(v_vars, DOSEQ)
        else:
            boundary, data = self.multipart_encode(v_vars, v_files)
            contenttype = 'multipart/form-data; boundary=' + str(boundary)
            if request.has_header('Content-Type') and \
               request.get_header('Content-Type').find( \
               'multipart/form-data') != 0:
                print "Replacing %s with %s" % \
                    (request.get_header('content-type'), \
                    'multipart/form-data')
            request.add_unredirected_header('Content-Type', contenttype)
            
        request.add_data(data)
        return request

    def multipart_encode(vars, files, boundary=None, buffer=None):
        if boundary is None:
            boundary = mimetools.choose_boundary()
        if buffer is None:
            buffer = ''
        for(key, value) in vars:
            buffer += '--%s\r\n' % boundary
            buffer += 'Content-Disposition: form-data; name="%s"' % key
            buffer += '\r\n\r\n' + value + '\r\n'
        for(key, fd) in files:
            file_size = os.fstat(fd.fileno())[stat.ST_SIZE]
            filename = fd.name.split('/')[-1]
            contenttype = mimetypes.guess_type(filename)[0] or \
                'application/octet-stream'
            buffer += '--%s\r\n' % boundary
            buffer += 'Content-Disposition: form-data; name="%s"; ' \
                'filename="%s"\r\n' % (key, filename)
            buffer += 'Content-Type: %s\r\n' % contenttype
            # buffer += 'Content-Length: %s\r\n' % file_size
            fd.seek(0)
            buffer += '\r\n' + fd.read() + '\r\n'
        buffer += '--%s--\r\n\r\n' % boundary
        return boundary, buffer
    multipart_encode = Callable(multipart_encode)

    https_request = http_request

